<?php

namespace Controller;

use Util\InstanceProvider;
use Util\Utilities;
use Util\Messages;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class Structure extends Simple {

    public function findStructures(Request\Filter $request, Response $response)
    {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $structures = InstanceProvider::getService()->getStructure()->getStructures($filter);
            $response->addDataSet(new DataSet('findStructures', $structures));
        } catch (\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function findStructureByResponsible(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $responsibleIds = null;
        if($filter){
            $responsibleIds = $filter[0]['value'];
            if(empty($responsibleIds)){
                $responsibleIds = null;
            }else if(is_array($responsibleIds) && !$responsibleIds[0]){
                $responsibleIds = null;
            }
        }
        $structures = InstanceProvider::getService()->getStructure()->findStructureByResponsible($responsibleIds);
        $response->addDataSet(new DataSet('findStructureByResponsible', $structures));
    }

    public function findStructure(Request\Filter $request, Response $response) {
        $structure = InstanceProvider::getService()->getStructure()->getStructure();
        $response->addDataSet(new DataSet('structures', $structure));
    }

    public function BasicCalendarSave(Request\DataSet $request, Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];

        $id = $row["ID"];
        $calendarDefault = $row["CALENDAR_DEFAULT"];
        $structure = $row["STRUCTURE"];
        $calendar = InstanceProvider::getService()->getStructure()->saveEditCalendar($id,$calendarDefault,$structure);
        $response->addDataSet(new DataSet('basicCalendarStructure', [$calendar->getId()]));

    }
    public function findBasicCalendar(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria()->getConditions();

        $structure = Utilities::getValueInFilterDataSourceByName($filter,'STRUCTURE');
        $calendars = InstanceProvider::getService()->getStructure()->findBasicCalendar($structure);
        $response->addDataSet(new DataSet('findBasicCalendar',($calendars)));

    }

    public function findStructuresParents(Request\Filter $request, Response $response) {
        $structures = $request->getFilterCriteria()->getConditions();
        $structuresParents = InstanceProvider::getService()->getStructure()->getStructuresParents($structures);
        $response->addDataSet(new DataSet('structuresParents', $structuresParents));
    }

    public function findStructureInformation(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $structure = $filter[0]['value'];       

        $structureInformation = InstanceProvider::getService()->getStructure()->getStructureInformation($structure);
        $response->addDataSet(new DataSet("Basic", $structureInformation));
    }

    public function findEditStructure(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $structure = $filter[0]['value'];

        $editStructure = InstanceProvider::getService()->getStructure()->getEditStructure($structure);
        $response->addDataset(new DataSet("Basic", $editStructure));
    }

    public function findStructureTypes(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $structureType = InstanceProvider::getService()->getStructure()->getStructureTypes($filter);
        $response->addDataSet(new DataSet('structureType', $structureType));
    }

    public function findBusinessPartnerForStructure(Request\Filter $request, Response $response) {
        $businessPartner = InstanceProvider::getService()->getStructure()->getBusinessPartner();
        $response->addDataset(new DataSet('businessPartner', $businessPartner));
    }

    public function findBusinessPartnerForEditStructure(Request\Filter $request, Response $response) {
        $businessPartner = InstanceProvider::getService()->getStructure()->getBusinessPartnerForEditStructure();
        $response->addDataset(new DataSet('businessPartnerEdit', $businessPartner));
    }

    public function saveStructure(Request\DataSet $request, Response $response)
    {
        try {
            $data = $request->getDataSet()->getRows();
            $param = $data[0];
            $structureId = empty($param['ID']) ? null : $param['ID'];
            $name = $param['NAME'];
            $structureType = $param['STRUCTURE_TYPE'];
            $resource = empty($param['BUSINESS_PARTNER']) ? null : $param['BUSINESS_PARTNER'];
            $parent = empty($param['PARENT']) ? null : $param['PARENT'];
            $status = $param['STATUS'];
            $timeSheetRequired = empty($param['TIME_SHEET_REQUIRED']) ? null : $param['TIME_SHEET_REQUIRED'];
            $replicateTimeSheetRequired = empty($param['REPLICATE_TIME_SHEET_REQUIRED']) ? false : $param['REPLICATE_TIME_SHEET_REQUIRED'];
            $savedStructure = InstanceProvider::getService()->getStructure()->saveStructure($structureId, $name, $structureType, $resource, $parent, $status,
                $timeSheetRequired, $replicateTimeSheetRequired);
            $response->addDataset(new DataSet('structure', $savedStructure));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function removeStructures($request, Response $response) {
        if($request instanceof Request\DataSet){
            $dataSet = $request->getDataSet()->getRows();
        }else{
            $dataSet = array( $request->getRow());
        }

        foreach ($dataSet as $param) {
            try{
                InstanceProvider::getService()->getStructure()->deleteStructure($param['ID']);
                $response->addNotification(new Response\Notification(Messages::SUC_REMOVED_STRUCTURES));
            }catch(\Exception $ex){
                throw new \Exception($ex->getMessage());
            }
        }
    }

    public function saveStructureType(Request\DataSet $request, Response $response) {
        $data = $request->getDataSet()->getRows();
        $param = $data[0];
        $label = $param['LABEL'];
        try{
            InstanceProvider::getService()->getStructureType()->createStructureType($label);
            $response->addMessage(new Response\Message(\Util\Messages::SUC_SUCCESSFUL_SAVED));
        }catch(\Exception $ex){
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
        }
    }
    
    public function saveCalendarDetail(Request\DataSet $request, Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        foreach ($rows as &$row) {
            list($recurrenceType, $calendar, $id, $type, $status, $name, $beginDate, $occurrences,
                $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
                $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime)
                = CalendarDetail::builderExceptionRow($row);

            $calendarDetail = InstanceProvider::getService()->getStructure()->saveException($id,$recurrenceType,$name,$calendar,$type,$status,$beginDate,$occurrences,
                $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
                $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime);
        }
        $response->addDataset(new DataSet('calendarDetail', array("ID" => $calendarDetail->getId())));
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
    }
    
    public function saveWeeklyDays(Request\DataSet $request, Response $response)
    {
        try{
            $rows = $request->getDataSet()->getRows()[0];

            list($calendar, $id, $typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday,
                $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday,
                $typeThursday, $dailyHoursThursday, $typeFriday, $dailyHoursFriday,
                $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate, $name) =
                CalendarWeeklyDays::buidingCreateWeekWork($rows);

            $weeklyDays = InstanceProvider::getService()->getStructure()->saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
                
            $response->addDataset(new DataSet('CalendarDetail', array("ID" => $weeklyDays->getId())));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        }catch(\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado :'.$e->getMessage(),Response::STATUS_ERROR));
        }
    }

    public function findStructuresByManagerOn(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $structures = InstanceProvider::getService()->getStructure()->getStructuresFromManagerOnline($filter);
        $response->addDataSet(new DataSet('structure', $structures));
    }
    public function findGroup(Request\Filter $request,Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $manager = Utilities::getValueInFilterDataSourceByName($filter,'PROJECT_MANAGER');

        $dataSet = InstanceProvider::getService()->getStructure()->findGroupManager($manager);
        $response->addDataset(new DataSet('groupProject', $dataSet));
    }
    public function findGroupEditProject(Request\Filter $request,Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $manager = Utilities::getValueInFilterDataSourceByName($filter,'PROJECT_MANAGER', true);
        try{
             $dataSet = InstanceProvider::getService()->getStructure()->findGroupManagerInEditProject($manager, $filter);
            $response->addDataset(new DataSet('groupEditProject', $dataSet));
        }catch(\Exception $ex){
               $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
        }
       
    }
    public function findCalendarDatesStructure(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $month = Utilities::getValueInFilterDataSourceByName($filters,'MONTH');
        $year = Utilities::getValueInFilterDataSourceByName($filters,'YEAR');
        $structureId = Utilities::getValueInFilterDataSourceByName($filters,'STRUCTURE');
        $dataSet = array();
        if($structureId){
            $dataSet = InstanceProvider::getService()->getStructure()->findExceptionCalendarDatesStructure($structureId,$month,$year);
        }
        $response->addDataset(new DataSet('CalendarDetail', $dataSet));
    }

} 