/***********************r************************************************************
****    Procedure para calculo do demonstrativo de resultado operacional       ******
*************************************************************************************

Criado em  : 06/2007
       Por : Alessandra Cristina

Descrio:

          - Com base nos dados parametrizados para a filial realiza apurao do
            resultado operacional da mesma, grava em tabelas especficas a fim de
            possibilitar a gerao de relatrios gerenciais a partir de dados
            previamente calculados.

Util :
       - Dir      : @C:\WEB_511\SQL\END_CONV\ORACLE\4420_GERSTPO_APURAC_RESOPERFILI.SQL

       - Erro     : raise_application_error(-20501, 'Mensagem', True);

       - Executa  : EXEC APURAC_RESOPERFILI('0000000123','01/07/2006','31/07/2006');
       - Arquivos : _0481, _0565, _596
       }
*/
create or replace PROCEDURE APURAC_RESOPERFILI
                                (P_CDFILIAL       FILIAL.CDFILIAL%TYPE,
                                 P_DTINICIAL      IN DATE,
                                 P_DTFINAL        IN DATE,
                                 P_IDCONSANT      IN VARCHAR) /*CALCULA CONSUMO ANTERIOR*/

IS
/************** Variveis que recebem parametrizaes*******************/

   V_CDGRUPPROD      VARCHAR2(10);
   V_IDEFETIVOROP    VARCHAR2(1);
   V_IDCOMPRAROP     VARCHAR2(1);
   V_IDAJUSTEROP     VARCHAR2(1);

   V_IDIMPLANTROP    VARCHAR2(1);
   V_IDINVFINAROP    VARCHAR2(1);
   V_IDINVINICROP    VARCHAR2(1);
   V_IDRETIRADAROP   VARCHAR2(1);
   V_IDDESPDIVROP    VARCHAR2(1);
   V_IDDESPENTROP    VARCHAR2(1);
   V_IDDESPMOBROP    VARCHAR2(1);
   V_IDFATUROP       VARCHAR2(2);
   V_IDNFSAIDAROP    VARCHAR2(1);

   V_FILIALPAR       VARCHAR2(4);   /*Filial padro*/
   V_IDUTPADRROPFIL  VARCHAR2(1);   /*Filial padro*/
   V_IDUTPADRROP     VARCHAR2(1);   /*Filial padro*/

/***********************************************************************/

   V_NRAPURREL       VARCHAR2(20);
   V_CDPRODFIN       VARCHAR2(20);
   V_CDPRODINI       VARCHAR2(20);
   V_NRMOVIESTQ      VARCHAR2(20);
   V_DTLANCMOVI      DATE;
   V_DTLANCMOVIF     DATE;

   V_NRROPFILIAL     VARCHAR2(20);
   V_NRLANCROP       VARCHAR2(20);
   V_NRITLANCROP     VARCHAR2(20);
   V_NRSEQRELACIT    VARCHAR2(10);
   V_NRTIPOCUSTO     NUMBER;
   V_PRODUTO         VARCHAR2(20);
   V_NRSEQIT         VARCHAR2(10);
   V_CDFILISUBGR     VARCHAR2(04);
   V_IDSERVRETROP    CHAR;
   V_IDSERVFATROP    CHAR;
   /*Variveis para calculo do consumo*/

   V_CONSUMO         FLOAT;
   V_NRCOUNT         FLOAT;
   V_NRCONS          FLOAT;
   V_VALOR           FLOAT;
   V_DTINICONS       DATE;

   V_NRMESCONSROP    NUMBER;
   V_IDPERCONSROP    CHAR;
   V_DTINICIO        DATE;
   V_DTINICIOST VARCHAR2(30);
   V_DTFINAL         DATE;

   V_DTFILIALROP     DATE;
   V_NRITEM          NUMBER;
   V_CDFILCENTPME    VARCHAR2(4);
   V_IDFILIALAGRROP  CHAR;

   V_IDFECHDIAROP    CHAR;
   V_APURCONS        VARCHAR2(10);
   V_DTLANCROP       DATE;
   V_NRROPFILINI     VARCHAR2(20);
   V_CDFILIALIT      VARCHAR2(04);
   V_NRLANCTONFIT    VARCHAR2(10);
   V_CDFISCALIT      CHAR;
   V_ULTLANCMOVIEST  VARCHAR2(20);
   V_VRCUSTLOGLAN    FLOAT;
   V_VRSEGULOGLAN    FLOAT;
   V_DTINIC          DATE;
   V_FILIAPURA       VARCHAR2(4);
   V_NRDIAS          FLOAT;
   V_DTPROCESSA      DATE;
   V_IDLANCDTMENFIL  VARCHAR2(2);
   V_PRIIMP          NUMBER;
   V_NRROPDIF        BOOLEAN;
   V_CDFILIALBO      VARCHAR2(04);
   V_IDGERROPFOS     VARCHAR2(1);
   V_IDPERFIXCALCIMP VARCHAR2(1);
   V_NRLANCTONF      NOTAFISCAL.NRLANCTONF%TYPE;
   V_COUNT           FLOAT;
   V_IDCONSFATUDIV   VARCHAR2(1);
   V_DTULTAPURACAO   DATE;
   V_DTALTEFETIVO    DATE;
   V_IDOBRINFNRFUNMO VARCHAR2(1);
   V_VRVENDACANCEL   FLOAT;
   V_ITVENDACAN      FLOAT;
   V_VRCANCELAMENTO  FLOAT;
   V_VENDABRUTA      FLOAT;
   V_VRCONTABIL      FLOAT;
BEGIN
   V_CONSUMO    := 0;
   V_NRLANCROP  := 0;
   V_NRDIAS     := 0;
   V_VRVENDACANCEL  := 0;
   V_VRVENDACANCEL  := 0;
   V_ITVENDACAN     := 0;
   V_VRCANCELAMENTO := 0;
   V_VENDABRUTA     := 0;
   V_VRCONTABIL     := 0;

   /*Verifica parametrizao para gerao do ROP*/

   BEGIN
      SELECT IDUTPADRROP
      INTO V_IDUTPADRROP
      FROM PARAMGERAL G;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_IDUTPADRROP     := NULL;
      END;
   END;

   BEGIN
      SELECT IDUTPADRROPFIL, CDFILCENTPME
      INTO V_IDUTPADRROPFIL, V_CDFILCENTPME
      FROM PARFILIALROP P, FILIAL F
      WHERE
         F.CDFILIAL    = P_CDFILIAL    AND
         P.CDFILIAL(+) = F.CDFILIAL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_IDUTPADRROPFIL := NULL;
         V_CDFILCENTPME   := NULL;
      END;
   END;


   IF (V_IDUTPADRROP = 'S' OR V_IDUTPADRROPFIL = 'S') THEN
      V_FILIALPAR := 'P';
   ELSE
      IF V_CDFILCENTPME IS NULL THEN
         V_FILIALPAR := P_CDFILIAL;
      ELSE
         BEGIN
            SELECT IDFILIALAGRROP
            INTO V_IDFILIALAGRROP
            FROM PARFILIALROP
            WHERE CDFILIAL = V_CDFILCENTPME;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_IDFILIALAGRROP := 'N';
         END;
         IF V_IDFILIALAGRROP = 'S' THEN
            V_FILIALPAR := V_CDFILCENTPME;
         ELSE
            V_FILIALPAR := P_CDFILIAL;
         END IF;
      END IF;
   END IF;

   BEGIN
      SELECT
          CDGRUPPROD,   IDFATUROP,    IDEFETIVOROP, IDCOMPRAROP,IDNFSAIDAROP,
          IDINVINICROP, IDIMPLANTROP, IDINVFINAROP, IDAJUSTEROP, IDRETIRADAROP,
          IDDESPDIVROP, IDDESPMOBROP, IDDESPENTROP, IDSERVRETROP, IDSERVFATROP,
          NVL(NRMESCONSROP,1), NVL(IDPERCONSROP, 'N'), IDFECHDIAROP, IDGERROPFOS, IDPERFIXCALCIMP, IDCONSFATUDIV
      INTO   V_CDGRUPPROD,    V_IDFATUROP,     V_IDEFETIVOROP, V_IDCOMPRAROP, V_IDNFSAIDAROP,
             V_IDINVINICROP,  V_IDIMPLANTROP,  V_IDINVFINAROP, V_IDAJUSTEROP, V_IDRETIRADAROP,
             V_IDDESPDIVROP,  V_IDDESPMOBROP,  V_IDDESPENTROP, V_IDSERVRETROP,V_IDSERVFATROP,
             V_NRMESCONSROP,  V_IDPERCONSROP,  V_IDFECHDIAROP, V_IDGERROPFOS, V_IDPERFIXCALCIMP, V_IDCONSFATUDIV
      FROM PARFILIALROP P
      WHERE P.CDFILIAL = V_FILIALPAR;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_CDGRUPPROD   := NULL;  V_IDFATUROP     := NULL;  V_IDEFETIVOROP := NULL; V_IDCOMPRAROP  := NULL;
         V_IDNFSAIDAROP := NULL;  V_IDINVINICROP  := NULL;  V_IDIMPLANTROP := NULL; V_IDINVFINAROP := NULL;
         V_IDAJUSTEROP  := NULL;  V_IDRETIRADAROP := NULL;  V_IDDESPDIVROP := NULL; V_IDDESPMOBROP := NULL;
         V_IDDESPENTROP := NULL;  V_IDSERVRETROP  := NULL;  V_IDSERVFATROP := NULL; V_NRMESCONSROP := 1;
         V_IDPERCONSROP := 'N';   V_IDFECHDIAROP  := 'N';   V_IDGERROPFOS := 'N';   V_IDPERFIXCALCIMP := 'N';
         V_IDCONSFATUDIV := 'N';
      END;
   END;

   BEGIN
      SELECT TO_NUMBER(NVL(CDTIPCUSTEST,'3')),
             DECODE(IDUTILDTPROC,'S',NVL(TRUNC(DTPROCESSA),TRUNC(SYSDATE)),TRUNC(SYSDATE)),
             IDOBRINFNRFUNMO
        INTO V_NRTIPOCUSTO, V_DTPROCESSA,V_IDOBRINFNRFUNMO
        FROM PARAMFILIAL P, PARAMGERAL
       WHERE P.CDFILIAL = P_CDFILIAL;

   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_NRTIPOCUSTO   := '3' ;
         V_DTPROCESSA    := TRUNC(SYSDATE);
   END;
   /*Verifica se filial tem sub-grupo especfico*/
   FOR C_FILGRUPO IN(SELECT DISTINCT CDFILISUBGR
                       FROM PRSUBGRPROD
                      WHERE CDGRUPPROD = V_CDGRUPPROD
                      ORDER BY CDFILISUBGR ASC)LOOP

          IF C_FILGRUPO.CDFILISUBGR = P_CDFILIAL THEN
             V_CDFILISUBGR := P_CDFILIAL;
             EXIT;
          ELSE
             V_CDFILISUBGR := 'P';
          END IF;
   END LOOP;

   /*Verifica se a data compreendida no perodo parametrizado j foi apurada*/

   IF P_IDCONSANT = 'N' THEN

      V_FILIAPURA := V_FILIALPAR;
      IF V_FILIAPURA = 'P' THEN
         V_FILIAPURA := P_CDFILIAL;
      END IF;

      BEGIN
         SELECT DTINIAPURACROP
           INTO V_DTINIC
           FROM APURACAOROP
          WHERE CDFILIAL = V_FILIAPURA;

      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_DTINIC   := P_DTINICIAL;
      END;

      BEGIN
         SELECT NVL(IDLANCDTMENFIL,'N')
           INTO V_IDLANCDTMENFIL
           FROM PARAMFILIAL
          WHERE CDFILIAL = P_CDFILIAL;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_IDLANCDTMENFIL := 'N';
      END;

      IF NVL(V_IDLANCDTMENFIL,'N') = 'S' THEN
         BEGIN
            SELECT MAX(NRDIAS)
              INTO V_NRDIAS
              FROM (SELECT NRDIASENTRFIL NRDIAS
                      FROM PARAMFILIAL
                     WHERE CDFILIAL = P_CDFILIAL
                       AND NVL(IDLANCDTMENFIL,'N') = 'S'
                     UNION ALL
                    SELECT NRDIASSAIDFIL NRDIAS
                      FROM PARAMFILIAL
                     WHERE CDFILIAL = P_CDFILIAL
                       AND NVL(IDLANCDTMENFIL,'N') = 'S'
                     UNION ALL
                    SELECT NRDIASAJUSFIL NRDIAS
                      FROM PARAMFILIAL
                     WHERE CDFILIAL = P_CDFILIAL
                       AND NVL(IDLANCDTMENFIL,'N') = 'S') A;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NRDIAS := 0;
         END;

      ELSE

         BEGIN
            SELECT MAX(NRDIAS)
              INTO V_NRDIAS
              FROM (SELECT NRDIASENTR NRDIAS
                      FROM PARAMESTOQUE
                     WHERE NVL(IDLANCDTMEN,'N') = 'S'
                     UNION ALL
                    SELECT NRDIASSAID NRDIAS
                      FROM PARAMESTOQUE
                     WHERE NVL(IDLANCDTMEN,'N') = 'S'
                     UNION ALL
                    SELECT NRDIASAJUS NRDIAS
                      FROM PARAMESTOQUE
                     WHERE NVL(IDLANCDTMEN,'N') = 'S')A;

         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NRDIAS := 0;
         END;

      END IF;

      IF (V_DTINIC >= V_DTPROCESSA - V_NRDIAS) THEN
         V_DTINIC := V_DTPROCESSA - V_NRDIAS;
      END IF;

      IF NOT(V_DTINIC < P_DTINICIAL AND V_DTINIC IS NOT NULL) THEN
         V_DTINIC := P_DTINICIAL;
      END IF;

/* Verifica se a data inicial  maior que a data final da ultima apurao. A apurao deve ser feita a partir da ultima
para que no haja buracos. */
      BEGIN
         SELECT DTINIAPURACROP
           INTO V_DTULTAPURACAO
           FROM APURACAOROP
          WHERE CDFILIAL = V_FILIAPURA;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_DTULTAPURACAO := NULL;
      END;

      IF V_DTULTAPURACAO IS NOT NULL THEN
         IF V_DTULTAPURACAO < V_DTINIC THEN
            V_DTINIC := V_DTULTAPURACAO;
         END IF;
      END IF;

      FOR C_LANCROPFILI IN(SELECT CDFILIAL, NRROPFILIAL, NRLANCROP
                         FROM LANCROPFILIAL
                         WHERE CDFILIAL     =  P_CDFILIAL       AND
                               DTLANCROP  >= V_DTINIC) LOOP

         DELETE ITRELACITROP
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRLANCROP   = C_LANCROPFILI.NRLANCROP AND
               NRROPFILIAL = C_LANCROPFILI.NRROPFILIAL;

         DELETE ITLANCROPFILIAL
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRLANCROP   = C_LANCROPFILI.NRLANCROP AND
               NRROPFILIAL = C_LANCROPFILI.NRROPFILIAL;

         DELETE LANCROPFILIAL
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRLANCROP   = C_LANCROPFILI.NRLANCROP AND
               NRROPFILIAL = C_LANCROPFILI.NRROPFILIAL;
      END LOOP;

      FOR C_ROPFILI IN(SELECT NRROPFILIAL
                         FROM ROPFILIAL
                         WHERE CDFILIAL     =  P_CDFILIAL       AND
                               DTFILIALROP  >= V_DTINIC) LOOP

         DELETE ROPFILIAL
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRROPFILIAL = C_ROPFILI.NRROPFILIAL;

      END LOOP;
   ELSE
      V_DTINIC := P_DTINICIAL;
   END IF;

   V_DTFILIALROP := V_DTINIC;


   NOVO_CODIGO('RESULTOPER' || P_CDFILIAL, V_NRROPFILIAL, ROUND(TO_NUMBER(P_DTFINAL - V_DTINIC)+10,0));
   V_NRROPFILIAL := SUBSTR(V_NRROPFILIAL, 11, 10);
   V_NRROPFILINI := V_NRROPFILIAL;


   WHILE (V_DTFILIALROP <= P_DTFINAL) LOOP
      INSERT INTO ROPFILIAL
         (CDFILIAL, NRROPFILIAL, DTINICROP, DTFINALROP, DTGERAROP, DTFILIALROP)
      VALUES
         (P_CDFILIAL, V_NRROPFILIAL, V_DTINIC, P_DTFINAL, SYSDATE, V_DTFILIALROP);

      /*Grava Posio Incial e Final para cada dia apurado*/

      /*Inventrio Inicial*/

      IF V_IDINVINICROP = 'S' THEN
         V_NRITLANCROP := '00000';
         IF V_NRTIPOCUSTO IN (40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,57,58,59,60,61,62) THEN
            FOR C_INVINICIAL IN(SELECT AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                       SUM(QTESTDIACON) AS QTESTDIACON, SUM(VRESTDIACON) AS VRESTDIACON
                                FROM GRUPPROD GR, SUBGRPRODFIL SG,
                                   PARFILIALROP PA, PRSUBGRPROD AN, PRODUTO A,
                                   (SELECT CDPRODUTO, QTESTDIACON,
                                           NVL(CUSTO_PRODUTO(P.CDFILIAL, P.CDPRODUTO, V_NRTIPOCUSTO, V_DTINIC, SYSDATE),0) * QTESTDIACON AS VRESTDIACON
                                     /* Alterao realizada para buscar o custo parametrizado */
                                    FROM POSESTCONS P
                                    WHERE (P.CDFILIAL   = P_CDFILIAL)               AND
                                          (QTESTDIACON <> 0  OR VRESTDIACON <> 0)   AND
                                          (P.DTPOSESTCON = (SELECT Max(DTPOSESTCON)
                                                            FROM POSESTCONS C
                                                            WHERE (C.CDFILIAL     = P_CDFILIAL)     AND
                                                                  (C.CDPRODUTO    = P.CDPRODUTO)    AND
                                                                  (C.DTPOSESTCON <= V_DTFILIALROP)))) P
                              WHERE
                                 (SG.CDFILISUBGR     = V_CDFILISUBGR)         AND
                                 (PA.CDFILIAL     = V_FILIALPAR)           AND
                                 (PA.CDGRUPPROD   = GR.CDGRUPPROD)         AND
                                 (GR.CDGRUPPROD   = SG.CDGRUPPROD)         AND
                                 (SG.CDSUBGRPROD  = AN.CDSUBGRPROD)        AND
                                 (P.CDPRODUTO     = AN.CDPRODUTO)          AND
                                 (SG.CDGRUPPROD   = AN.CDGRUPPROD)         AND
                                 (P.CDPRODUTO     = A.CDPRODUTO)           AND
                                 (A.CDPRODUTO     = A.CDPRODESTO)
                             GROUP BY AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                             ORDER BY AN.CDGRUPPROD, AN.CDSUBGRPROD DESC) LOOP


               V_NRLANCROP := LPAD(V_NRLANCROP + 1,10,'0');

               /*Inclui Inventario Inicial*/
               INSERT INTO LANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                   NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
               VALUES
                  (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, V_DTFILIALROP, 'II', '',
                   '',  '', '', P_CDFILIAL,'', V_CDFILCENTPME);

               /*Inclui Item do Lanamento*/
               V_NRITLANCROP := '00001';

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, P_CDFILIAL, '',
                   '', C_INVINICIAL.CDSUBGRPROD,  C_INVINICIAL.QTESTDIACON, C_INVINICIAL.VRESTDIACON,
                   C_INVINICIAL.CDGRUPPROD, C_INVINICIAL.CDSUBGRPROD, C_INVINICIAL.IDSUBGRMATPRIMA);

               V_CONSUMO := V_CONSUMO + C_INVINICIAL.VRESTDIACON;

            END LOOP;
         ELSE
            FOR C_INVINICIAL IN(SELECT AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                       SUM(QTESTDIACON) AS QTESTDIACON, SUM(VRESTDIACON) AS VRESTDIACON
                                  FROM SUBGRPRODFIL SG, PRSUBGRPROD AN, PRODUTO A,
                                   (SELECT CDPRODUTO, QTESTOQDIA AS QTESTDIACON,
                                           NVL(CUSTO_PRODUTO_LCEST(B.CDFILIAL, B.CDPRODUTO, V_NRTIPOCUSTO, V_DTFILIALROP, SYSDATE, CDALMOXARIFE, CDLOCALESTOQ,  B.NRLOTEESTQ, B.NRSUBLOTE),0) *QTESTOQDIA AS VRESTDIACON
                                      FROM POSESTDIA B
                                     WHERE (B.CDFILIAL   = P_CDFILIAL)         AND
                                           (B.QTESTOQDIA  <> 0)              AND
                                           (B.DTPOSIESTQ =  (SELECT MAX(P.DTPOSIESTQ)
                                                              FROM POSESTDIA P
                                                             WHERE (P.CDFILIAL       = P_CDFILIAL)      AND
                                                                    (P.CDPRODUTO     = B.CDPRODUTO)     AND
                                                                    (P.CDALMOXARIFE  = B.CDALMOXARIFE)  AND
                                                                    (P.CDLOCALESTOQ  = B.CDLOCALESTOQ)  AND
                                                                    (P.NRLOTEESTQ    = B.NRLOTEESTQ)    AND
                                                                    (P.NRSUBLOTE     = B.NRSUBLOTE)     AND
                                                                    (P.DTPOSIESTQ    <= V_DTFILIALROP)))) P

                                WHERE
                                 (SG.CDFILISUBGR  = V_CDFILISUBGR) AND
                                 (SG.CDGRUPPROD   = V_CDGRUPPROD ) AND
                                 (AN.CDFILISUBGR = SG.CDFILISUBGR)  AND
                                 (AN.CDGRUPPROD  = SG.CDGRUPPROD  ) AND
                                 (AN.CDSUBGRPROD = SG.CDSUBGRPROD)  AND
                                 (P.CDPRODUTO    = AN.CDPRODUTO)    AND
                                 (P.CDPRODUTO    = A.CDPRODUTO)     AND
                                 (A.CDPRODUTO   = A.CDPRODESTO)
                             GROUP BY AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                             ORDER BY AN.CDGRUPPROD, AN.CDSUBGRPROD DESC) LOOP

               V_NRLANCROP := LPAD(V_NRLANCROP + 1,10,'0');

               /*Inclui Inventario Inicial*/
               INSERT INTO LANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                   NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
               VALUES
                  (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, V_DTFILIALROP, 'II', '',
                   '',  '', '', P_CDFILIAL,'', V_CDFILCENTPME);


               /*Inclui Item do Lanamento*/
               V_NRITLANCROP := '00001';

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, P_CDFILIAL, '',
                   '', C_INVINICIAL.CDSUBGRPROD,  C_INVINICIAL.QTESTDIACON, C_INVINICIAL.VRESTDIACON,
                   C_INVINICIAL.CDGRUPPROD, C_INVINICIAL.CDSUBGRPROD, C_INVINICIAL.IDSUBGRMATPRIMA);

               V_CONSUMO := V_CONSUMO + C_INVINICIAL.VRESTDIACON;

            END LOOP;
         END IF;
      END IF;

                                               /* Alterao realizada para buscar o custo parametrizado */
      /*Inventrio Final*/
      IF V_IDINVFINAROP = 'S' THEN
         V_NRITLANCROP := '00000';
         IF V_NRTIPOCUSTO IN (40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,57,58,59,60,61,62) THEN
            FOR C_INVFINAL IN(SELECT AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                     SUM(QTESTDIACON) AS QTESTDIACON, SUM(VRESTDIACON) AS VRESTDIACON
                                FROM GRUPPROD GR, SUBGRPRODFIL SG,
                                   PARFILIALROP PA, PRSUBGRPROD AN, PRODUTO A,
                                   (SELECT CDPRODUTO, QTESTDIACON,
                                   NVL(CUSTO_PRODUTO(P.CDFILIAL, P.CDPRODUTO, V_NRTIPOCUSTO, P_DTFINAL, SYSDATE),0) * QTESTDIACON AS VRESTDIACON
                                            /* Alterao realizada para buscar o custo parametrizado */
                                    FROM POSESTCONS P
                                    WHERE (P.CDFILIAL   = P_CDFILIAL)               AND
                                          (QTESTDIACON <> 0  OR VRESTDIACON <> 0)   AND
                                          (P.DTPOSESTCON = (SELECT Max(DTPOSESTCON)
                                                            FROM POSESTCONS C
                                                            WHERE (C.CDFILIAL     = P_CDFILIAL)     AND
                                                                  (C.CDPRODUTO    = P.CDPRODUTO)    AND
                                                                  (C.DTPOSESTCON <= V_DTFILIALROP+1)))) P
                              WHERE
                                 (SG.CDFILISUBGR  = V_CDFILISUBGR)         AND
                                 (PA.CDFILIAL     = V_FILIALPAR)           AND
                                 (PA.CDGRUPPROD   = GR.CDGRUPPROD)         AND
                                 (GR.CDGRUPPROD   = SG.CDGRUPPROD)         AND
                                 (SG.CDSUBGRPROD  = AN.CDSUBGRPROD)        AND
                                 (P.CDPRODUTO     = AN.CDPRODUTO)          AND
                                 (SG.CDGRUPPROD   = AN.CDGRUPPROD)         AND
                                 (P.CDPRODUTO     = A.CDPRODUTO)           AND
                                 (A.CDPRODUTO     = A.CDPRODESTO)
                             GROUP BY AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                             ORDER BY AN.CDGRUPPROD, AN.CDSUBGRPROD DESC) LOOP

               /*Inclui Inventario FINAL*/
               V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
               INSERT INTO LANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                   NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
               VALUES
                  (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, V_DTFILIALROP, 'IF', '',
                   '',  '', '', P_CDFILIAL,'', V_CDFILCENTPME);

               /*Inclui Item do Lanamento*/
               V_NRITLANCROP := '00001';

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, P_CDFILIAL, '',
                   '', C_INVFINAL.CDSUBGRPROD,  C_INVFINAL.QTESTDIACON, C_INVFINAL.VRESTDIACON,
                   C_INVFINAL.CDGRUPPROD, C_INVFINAL.CDSUBGRPROD, C_INVFINAL.IDSUBGRMATPRIMA);

               V_CONSUMO := V_CONSUMO - C_INVFINAL.VRESTDIACON;
            END LOOP;
         ELSE
            FOR C_INVFINAL IN(SELECT AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                     SUM(QTESTDIACON) AS QTESTDIACON, SUM(VRESTDIACON) AS VRESTDIACON
                                FROM SUBGRPRODFIL SG, PRSUBGRPROD AN, PRODUTO A,
                                   (SELECT CDPRODUTO, QTESTOQDIA AS QTESTDIACON,
                                           NVL(CUSTO_PRODUTO_LCEST(B.CDFILIAL, B.CDPRODUTO, V_NRTIPOCUSTO, V_DTFILIALROP, SYSDATE, CDALMOXARIFE, CDLOCALESTOQ,  B.NRLOTEESTQ, B.NRSUBLOTE),0) *QTESTOQDIA AS VRESTDIACON
                                      FROM POSESTDIA B
                                     WHERE (B.CDFILIAL   = P_CDFILIAL)         AND
                                           (B.QTESTOQDIA  <> 0)              AND
                                           (B.DTPOSIESTQ =  (SELECT MAX(P.DTPOSIESTQ)
                                                              FROM POSESTDIA P
                                                             WHERE (P.CDFILIAL       = P_CDFILIAL)      AND
                                                                    (P.CDPRODUTO     = B.CDPRODUTO)     AND
                                                                    (P.CDALMOXARIFE  = B.CDALMOXARIFE)  AND
                                                                    (P.CDLOCALESTOQ  = B.CDLOCALESTOQ)  AND
                                                                    (P.NRLOTEESTQ    = B.NRLOTEESTQ)    AND
                                                                    (P.NRSUBLOTE     = B.NRSUBLOTE)     AND
                                                                    (P.DTPOSIESTQ    <= V_DTFILIALROP+1)))) P
                              WHERE
                                 (SG.CDFILISUBGR  = V_CDFILISUBGR)AND
                                 (SG.CDGRUPPROD   = V_CDGRUPPROD )AND
                                 (AN.CDFILISUBGR = SG.CDFILISUBGR)AND
                                 (AN.CDGRUPPROD = SG.CDGRUPPROD  )AND
                                 (AN.CDSUBGRPROD = SG.CDSUBGRPROD)AND
                                 (P.CDPRODUTO    = A.CDPRODUTO)   AND
                                 (P.CDPRODUTO     = AN.CDPRODUTO) AND
                                 (A.CDPRODUTO     = A.CDPRODESTO)
                             GROUP BY AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                             ORDER BY AN.CDGRUPPROD, AN.CDSUBGRPROD DESC) LOOP

               /*Inclui Inventario FINAL*/
               V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
               INSERT INTO LANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                   NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
               VALUES
                  (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, V_DTFILIALROP, 'IF', '',
                   '',  '', '', P_CDFILIAL,'', V_CDFILCENTPME);

               /*Inclui Item do Lanamento*/
               V_NRITLANCROP := '00001';

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, P_CDFILIAL, '',
                   '', C_INVFINAL.CDSUBGRPROD,  C_INVFINAL.QTESTDIACON, C_INVFINAL.VRESTDIACON,
                   C_INVFINAL.CDGRUPPROD, C_INVFINAL.CDSUBGRPROD, C_INVFINAL.IDSUBGRMATPRIMA);

               V_CONSUMO := V_CONSUMO - C_INVFINAL.VRESTDIACON;
            END LOOP;
         END IF;
      END IF;

      V_DTFILIALROP  := TO_DATE(V_DTFILIALROP+1);
      V_NRROPFILIAL  := LPAD(V_NRROPFILIAL + 1,10,'0');
   END LOOP;

   /* Clculo do Faturamento*/

   /*Faturamento Bruto - Notas Fiscais de sada */
   IF V_IDFATUROP = 'FS' OR V_IDFATUROP = 'AB' THEN
      V_DTLANCROP := '01/01/1990';
      FOR C_FATURANF IN(SELECT N.CDCLIENTE,    N.CDTIPOOPER, N.NRNOTAFISC, N.CDFILIAL,
                               N.NRLANCTONF,   N.DTENTRSAID, N.CDSERIE,    N.IDSTATUS,
                               N.DTINIREFEFAT, N.DTFINREFEFAT,
                               DECODE(IDSISTEMA, 'PDV', N.VRPRODUTOS, N.VRNOTAFISC) AS VALORTOTAL
                          FROM NOTAFISCAL N, TIPOOPERACROP P
                         WHERE (N.CDFILIEMIT    = P_CDFILIAL)
                           AND (N.IDENTRSAIDOP  = 'S')
                           AND (N.DTENTRSAID BETWEEN V_DTINIC -15 AND P_DTFINAL)
                           AND (V_FILIALPAR     = P.CDFILIAL)
                           AND (N.CDTIPOOPER    = P.CDTIPOOPER)
                           AND (N.IDENTRSAIDOP  = P.IDENTRSAIDOP)
                           AND (P.IDTPOPERACROP = 'FS')
                           AND ((N.IDSTATUS IN ('A','E'))OR(N.IDSTATUSNFE = 'A') OR (N.IDSISTEMA = 'PDV'))
                           AND (N.IDSISTEMA NOT IN ('REC','FIS') OR N.IDSISTEMA IS NULL)) LOOP

/* Verifica se a nota existe*/
         FOR C_REG IN( SELECT CDFILIAL, NRROPFILIAL,  NRLANCROP
                         FROM LANCROPFILIAL
                         WHERE
                            CDFILIAL = P_CDFILIAL AND
                            IDTPLANCROP = 'FS' AND
                            CDFILIALNFROP = C_FATURANF.CDFILIAL  AND
                            NRLANCTONFROP = C_FATURANF.NRLANCTONF )LOOP

            DELETE FROM ITRELACITROP
                  WHERE
                    CDFILIAL  = C_REG.CDFILIAL AND
                    NRROPFILIAL = C_REG.NRROPFILIAL AND
                    NRLANCROP = C_REG.NRLANCROP;

            DELETE FROM LANCROPFILIAL
                  WHERE
                     CDFILIAL  = C_REG.CDFILIAL AND
                     NRROPFILIAL = C_REG.NRROPFILIAL AND
                     NRLANCROP = C_REG.NRLANCROP;
         END LOOP;

         V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_FATURANF.DTENTRSAID THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_FATURANF.DTENTRSAID;

            V_DTLANCROP   := C_FATURANF.DTENTRSAID;

            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;
/* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

         /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP, CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP, IDSTATUSROP,
             DTINIREFEROP,  DTFINREFEROP)
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_FATURANF.DTENTRSAID, 'FS', C_FATURANF.CDCLIENTE,
             C_FATURANF.CDTIPOOPER,  C_FATURANF.NRNOTAFISC, C_FATURANF.CDSERIE, C_FATURANF.CDFILIAL,
             C_FATURANF.NRLANCTONF, V_CDFILCENTPME, C_FATURANF.IDSTATUS, C_FATURANF.DTINIREFEFAT,
             C_FATURANF.DTFINREFEFAT);

            /*Inclui Item do Lanamento*/
            V_NRITLANCROP := LPAD('00000' + 1,5,'0');

            INSERT INTO ITLANCROPFILIAL
               (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP)
            VALUES
               (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_FATURANF.CDFILIAL, C_FATURANF.NRLANCTONF,
                '', '',  0, C_FATURANF.VALORTOTAL);

         /*Imposto por Item NF */
         V_NRSEQRELACIT := 0;

         IF V_IDPERFIXCALCIMP = 'N' THEN
            FOR C_IMPOSTO IN(SELECT P.CDIMPOSTO,
                                    SUM(VALOR_IMPOSTO_NF(C_FATURANF.CDFILIAL, C_FATURANF.NRLANCTONF, CDIMPOSTO,'I')) AS VRIMPOSTO
                               FROM IMPOSTOROP P
                              WHERE P.CDFILIAL      = V_FILIALPAR
                                AND IDTPIMPOSTOROP  = 'FS'
                              GROUP BY P.CDIMPOSTO
                              ORDER BY P.CDIMPOSTO)LOOP

               V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

               INSERT INTO ITRELACITROP
                  (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                   CDITRELITROP, QTITRELITROP, VRITRELITROP)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                   C_IMPOSTO.CDIMPOSTO, 0,C_IMPOSTO.VRIMPOSTO);
            END LOOP;
         ELSE

            FOR C_IMPOSTO IN(SELECT P.CDIMPOSTO,
                                    (C_FATURANF.VALORTOTAL * NVL(VRPERFIXCALCIMP,0))/100 AS VRIMPOSTO,
                                    VRPERFIXCALCIMP
                               FROM IMPOSTOROP P
                              WHERE P.CDFILIAL      = V_FILIALPAR
                                AND IDTPIMPOSTOROP  = 'FS'
                              ORDER BY P.CDIMPOSTO)LOOP

               V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

     /* O CAMPO DE QUANTIDADE ARMAZENA O PERCENTUAL FIXO DE IMPOSTO UTILIZADO NO MOMENTO DA APURAO ESTE CAMPO SERVE COMO HISTRICO CASO O USURIO
      ALTERE O PERCENTUAL FIXO DO IMPOSTO. */
               INSERT INTO ITRELACITROP
                  (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                   CDITRELITROP, QTITRELITROP, VRITRELITROP)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                   C_IMPOSTO.CDIMPOSTO, C_IMPOSTO.VRPERFIXCALCIMP, C_IMPOSTO.VRIMPOSTO);
            END LOOP;
         END IF;
      END LOOP;
   END IF;

/*Faturamento Bruto - Medio De Efetivos */
   IF V_IDFATUROP = 'FE' OR V_IDFATUROP = 'AB' THEN
      SELECT LPAD(ABS(ROUND(DBMS_RANDOM.VALUE(9999,9999999999))),10,'0')
      INTO V_NRAPURREL
      FROM DUAL;

      SELECT MIN(DTALTEFETIVO)
        INTO V_DTALTEFETIVO
        FROM LOGALTEFET
       WHERE
          CDFILIAL = P_CDFILIAL AND
          IDATUALROP = 'N';

      IF (V_DTALTEFETIVO IS NULL)OR((V_DTALTEFETIVO IS NOT NULL) AND( V_DTALTEFETIVO > V_DTINIC)) THEN
         V_DTALTEFETIVO := V_DTINIC;
      END IF;

      FOR C_LANCROPFILI IN(SELECT CDFILIAL, NRROPFILIAL, NRLANCROP
                         FROM LANCROPFILIAL
                         WHERE CDFILIAL    =  P_CDFILIAL   AND
                               DTLANCROP  >= V_DTALTEFETIVO AND
                               IDTPLANCROP IN ('FE', 'EP')) LOOP

         DELETE ITRELACITROP
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRLANCROP   = C_LANCROPFILI.NRLANCROP AND
               NRROPFILIAL = C_LANCROPFILI.NRROPFILIAL;

         DELETE ITLANCROPFILIAL
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRLANCROP   = C_LANCROPFILI.NRLANCROP AND
               NRROPFILIAL = C_LANCROPFILI.NRROPFILIAL;

         DELETE LANCROPFILIAL
         WHERE CDFILIAL    = P_CDFILIAL  AND
               NRLANCROP   = C_LANCROPFILI.NRLANCROP AND
               NRROPFILIAL = C_LANCROPFILI.NRROPFILIAL AND
               IDTPLANCROP IN ('FE', 'EP');
      END LOOP;

      APURAC_RELGESDIRETA(P_CDFILIAL, V_DTALTEFETIVO, P_DTFINAL, V_NRAPURREL);
      V_DTLANCROP := '01/01/1990';
        /* Gesto Merenda  */
      SELECT CDFILCENTPME
        INTO V_CDFILCENTPME
        FROM FILIAL
       WHERE CDFILIAL = P_CDFILIAL;

      FOR C_EFET_PME IN (SELECT E.CDFILCENTPME, E.CDFILIAL, E.CDSERVFILI, E.DTEFETCLPME,
                                E.CDCLIENTE,
                                NVL(SUM(E.QTCOMEFCLPME),0) AS QTCOMEFCLPME,
                                NVL(SUM(VRCUSPADTCAR * E.QTCOMEFCLPME),0) VRDIETFATU
                           FROM EFETCLIEPME E, SERVICOROP S, CUSTPADTCARD C
                          WHERE E.CDFILCENTPME = V_CDFILCENTPME
                            AND E.CDFILIAL     = P_CDFILIAL
                            AND DTEFETCLPME  BETWEEN V_DTALTEFETIVO AND P_DTFINAL
                            AND V_FILIALPAR    = S.CDFILIAL
                            AND IDTPSERVICOROP = 'E'
                            AND V_IDSERVFATROP <> 'S'
                            AND E.CDSERVFILI = S.CDSERVFILI
                            AND E.CDFILCENTPME = C.CDFILCENTPME
                            AND E.CDSERVFILI = C.CDSERVFILI
                            AND S.CDSERVFILI = C.CDSERVFILI
                          GROUP BY E.CDFILCENTPME, E.CDFILIAL, E.CDSERVFILI,
                                   E.DTEFETCLPME, E.CDCLIENTE

                         UNION ALL

                         SELECT E.CDFILCENTPME, E.CDFILIAL, E.CDSERVFILI, E.DTEFETCLPME,
                                E.CDCLIENTE,
                                NVL(SUM(E.QTCOMEFCLPME),0) AS QTCOMEFCLPME,
                                NVL(SUM(VRCUSPADTCAR * E.QTCOMEFCLPME),0) VRDIETFATU
                           FROM EFETCLIEPME E, SERVICOROP S, CUSTPADTCARD C
                          WHERE E.CDFILCENTPME = V_CDFILCENTPME
                            AND E.CDFILIAL     = P_CDFILIAL
                            AND DTEFETCLPME  BETWEEN V_DTALTEFETIVO AND P_DTFINAL
                            AND V_IDSERVFATROP <> 'S'
                            AND E.CDFILCENTPME = C.CDFILCENTPME
                            AND E.CDSERVFILI = C.CDSERVFILI
                          GROUP BY E.CDFILCENTPME, E.CDFILIAL, E.CDSERVFILI,
                                   E.DTEFETCLPME, E.CDCLIENTE ) LOOP

         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_EFET_PME.DTEFETCLPME THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_EFET_PME.DTEFETCLPME;


            V_DTLANCROP   := C_EFET_PME.DTEFETCLPME;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;

            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

         /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_EFET_PME.DTEFETCLPME, 'FE', C_EFET_PME.CDCLIENTE,
             '',  '', '', C_EFET_PME.CDFILIAL, '', V_CDFILCENTPME);

         V_NRITLANCROP := '00000';

         /*Inclui Item do Lanamento*/
         V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

         INSERT INTO ITLANCROPFILIAL
            (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
             NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP)
         VALUES
            (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_EFET_PME.CDFILIAL, '',
             '', C_EFET_PME.CDSERVFILI, C_EFET_PME.QTCOMEFCLPME, C_EFET_PME.VRDIETFATU);



         V_NRSEQRELACIT := 0;
         IF V_IDPERFIXCALCIMP = 'N' THEN
            IMPOSTOROP_RELGESPME(P_CDFILIAL, V_DTINIC, P_DTFINAL, V_NRROPFILIAL,  V_NRLANCROP, V_NRITLANCROP, C_EFET_PME.CDCLIENTE,
                               C_EFET_PME.DTEFETCLPME, C_EFET_PME.CDSERVFILI);
         ELSE
            FOR C_IMPOSTO IN(SELECT P.CDIMPOSTO,
                                    (C_EFET_PME.VRDIETFATU * NVL(VRPERFIXCALCIMP,0))/100 AS VRIMPOSTO,
                                    VRPERFIXCALCIMP
                               FROM IMPOSTOROP P
                              WHERE P.CDFILIAL      = V_FILIALPAR
                                AND IDTPIMPOSTOROP  = 'FE'
                              ORDER BY P.CDIMPOSTO)LOOP

               V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

/* O CAMPO DE QUANTIDADE ARMAZENA O PERCENTUAL FIXO DE IMPOSTO UTILIZADO NO MOMENTO DA APURAO ESTE CAMPO SERVE COMO HISTRICO CASO O USURIO
   ALTERE O PERCENTUAL FIXO DO IMPOSTO. */
               INSERT INTO ITRELACITROP
                  (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                   CDITRELITROP, QTITRELITROP, VRITRELITROP)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                   C_IMPOSTO.CDIMPOSTO, C_IMPOSTO.VRPERFIXCALCIMP, C_IMPOSTO.VRIMPOSTO);
            END LOOP;
         END IF;
      END LOOP;

      /* Gestao Hospitalar */
      FOR C_EFET_HOSP IN (SELECT E.CDFILIAL, E.DTEFETDIET, E.CDSERVFILI,   E.CDCLIENTE,
                                 NVL(SUM(UTIL_APURAC.CalcValrServ(E.CDFILIAL, E.CDCLIENTE, E.CDSERVFILI,
                                                               DTEFETDIET, NULL) * E.QTCOMEFETDIE),0) AS VRDIETFATU,
                                 NVL(SUM(E.QTCOMEFETDIE),0) AS QTCOMEFETDIE
                            FROM EFETDIET E, SERVICOROP S
                           WHERE E.CDFILIAL     = P_CDFILIAL
                             AND E.CDSERVFILI   = S.CDSERVFILI
                             AND E.DTEFETDIET BETWEEN V_DTALTEFETIVO AND P_DTFINAL
                             AND V_FILIALPAR    = S.CDFILIAL
                             AND IDTPSERVICOROP = 'E'
                             AND V_IDSERVFATROP <> 'S'
                           GROUP BY E.CDFILIAL, E.DTEFETDIET, E.CDSERVFILI,   E.CDCLIENTE

                           UNION ALL

                          SELECT E.CDFILIAL, E.DTEFETDIET, E.CDSERVFILI,   E.CDCLIENTE,
                                 NVL(SUM(UTIL_APURAC.CalcValrServ(E.CDFILIAL, E.CDCLIENTE, E.CDSERVFILI,
                                                              DTEFETDIET, NULL) * E.QTCOMEFETDIE),0) AS VRDIETFATU,
                                 NVL(SUM(E.QTCOMEFETDIE),0) AS QTCOMEFETDIE
                            FROM EFETDIET E
                           WHERE E.CDFILIAL     = P_CDFILIAL
                             AND E.DTEFETDIET BETWEEN V_DTALTEFETIVO AND P_DTFINAL
                             AND V_IDSERVFATROP   = 'S'
                           GROUP BY E.CDFILIAL, E.DTEFETDIET, E.CDSERVFILI,   E.CDCLIENTE ) LOOP

         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_EFET_HOSP.DTEFETDIET THEN
            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_EFET_HOSP.DTEFETDIET;


            V_DTLANCROP   := C_EFET_HOSP.DTEFETDIET;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;
            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

         /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_EFET_HOSP.DTEFETDIET, 'FE', C_EFET_HOSP.CDCLIENTE,
             '',  '', '', C_EFET_HOSP.CDFILIAL, '', V_CDFILCENTPME);

         V_NRITLANCROP := '00000';

         /*Inclui Item do Lanamento*/
         V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

         INSERT INTO ITLANCROPFILIAL
            (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
             NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP)
         VALUES
            (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_EFET_HOSP.CDFILIAL, '',
             '', C_EFET_HOSP.CDSERVFILI,  C_EFET_HOSP.QTCOMEFETDIE, C_EFET_HOSP.VRDIETFATU);


         IF V_IDPERFIXCALCIMP  = 'S' THEN
            V_NRSEQRELACIT := 0;
            FOR C_IMPOSTO IN(SELECT P.CDIMPOSTO,
                                    (C_EFET_HOSP.VRDIETFATU * NVL(VRPERFIXCALCIMP,0))/100 AS VRIMPOSTO,
                                    VRPERFIXCALCIMP
                               FROM IMPOSTOROP P
                              WHERE P.CDFILIAL      = V_FILIALPAR
                                AND IDTPIMPOSTOROP  = 'FE'
                              ORDER BY P.CDIMPOSTO)LOOP

               V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

/* O CAMPO DE QUANTIDADE ARMAZENA O PERCENTUAL FIXO DE IMPOSTO UTILIZADO NO MOMENTO DA APURAO ESTE CAMPO SERVE COMO HISTRICO CASO O USURIO
   ALTERE O PERCENTUAL FIXO DO IMPOSTO. */
               INSERT INTO ITRELACITROP
                  (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                   CDITRELITROP, QTITRELITROP, VRITRELITROP)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                   C_IMPOSTO.CDIMPOSTO, C_IMPOSTO.VRPERFIXCALCIMP, C_IMPOSTO.VRIMPOSTO);
            END LOOP;
         ELSE
            IMPOSTOROP_RELGESHOSP(P_CDFILIAL, V_DTINIC, P_DTFINAL, V_NRROPFILIAL,  V_NRLANCROP, V_NRITLANCROP, C_EFET_HOSP.CDCLIENTE,
                                  C_EFET_HOSP.DTEFETDIET, C_EFET_HOSP.CDSERVFILI);
         END IF;

      END LOOP;

      FOR C_EFETIVOS IN (SELECT DISTINCT CDCLIENTEAPGD, DTAPURACFATGD,
                                CDSERVFILIAPGD, CDFILIALAPGD, NRAPURACGD,
                                NVL(SUM(T.NREFETIVOSAPGD),0) AS QUANTIDADE,
                                SUM(NVL(T.VRPRECUNITAPGD,0)) AS VRPRECUNITAPGD,
                                NVL(SUM(T.VRPRECUNITAPGD * T.NREFETIVOSAPGD),0) AS VALORTOTAL
                         FROM SERVICOROP S, TRAB_APURAC_GD T
                         WHERE (T.CDFILIALAPGD   = P_CDFILIAL)                               AND
                               (T.NRAPURACGD     = V_NRAPURREL)                              AND
                               (T.DTAPURACFATGD  BETWEEN V_DTALTEFETIVO  AND P_DTFINAL)         AND
                               (V_FILIALPAR      = S.CDFILIAL)                               AND
                               (T.CDSERVFILIAPGD = S.CDSERVFILI)                             AND
                               (IDTPSERVICOROP   = 'E')                                      AND
                               (V_IDSERVFATROP   <> 'S')
                        GROUP BY  CDFILIALAPGD, NRAPURACGD, T.CDSERVFILIAPGD, DTAPURACFATGD, CDCLIENTEAPGD

                         UNION ALL

                         SELECT DISTINCT CDCLIENTEAPGD, DTAPURACFATGD,
                                CDSERVFILIAPGD, CDFILIALAPGD, NRAPURACGD,
                                NVL(SUM(T.NREFETIVOSAPGD),0) AS QUANTIDADE,
                                SUM(NVL(T.VRPRECUNITAPGD,0)) AS VRPRECUNITAPGD,
                                NVL(SUM(T.VRPRECUNITAPGD * T.NREFETIVOSAPGD),0) AS VALORTOTAL
                         FROM TRAB_APURAC_GD T
                         WHERE (T.CDFILIALAPGD   = P_CDFILIAL)                               AND
                               (T.NRAPURACGD     = V_NRAPURREL)                              AND
                               (T.DTAPURACFATGD  BETWEEN V_DTALTEFETIVO  AND P_DTFINAL)         AND
                               (V_IDSERVFATROP   = 'S')

                         GROUP BY  CDFILIALAPGD, NRAPURACGD, T.CDSERVFILIAPGD, DTAPURACFATGD, CDCLIENTEAPGD) LOOP

         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_EFETIVOS.DTAPURACFATGD THEN
            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
              INTO V_NRROPFILIAL
             FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_EFETIVOS.DTAPURACFATGD;


            V_DTLANCROP   := C_EFETIVOS.DTAPURACFATGD;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;
            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;

         END IF;

         /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_EFETIVOS.DTAPURACFATGD, 'FE', C_EFETIVOS.CDCLIENTEAPGD,
             '',  '', '', C_EFETIVOS.CDFILIALAPGD, C_EFETIVOS.NRAPURACGD, V_CDFILCENTPME);

         V_NRITLANCROP := '00000';

         /*Inclui Item do Lanamento*/
         V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

         INSERT INTO ITLANCROPFILIAL
            (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
             NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP)
         VALUES
            (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_EFETIVOS.CDFILIALAPGD, C_EFETIVOS.NRAPURACGD,
             '', C_EFETIVOS.CDSERVFILIAPGD,  C_EFETIVOS.QUANTIDADE, C_EFETIVOS.VALORTOTAL);


         IF V_IDPERFIXCALCIMP  = 'S' THEN
            V_NRSEQRELACIT := 0;
            FOR C_IMPOSTO IN(SELECT P.CDIMPOSTO,
                                    (C_EFETIVOS.VALORTOTAL * NVL(VRPERFIXCALCIMP,0))/100 AS VRIMPOSTO,
                                    VRPERFIXCALCIMP
                               FROM IMPOSTOROP P
                              WHERE P.CDFILIAL      = V_FILIALPAR
                                AND IDTPIMPOSTOROP  = 'FE'
                              ORDER BY P.CDIMPOSTO)LOOP

               V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

/* O CAMPO DE QUANTIDADE ARMAZENA O PERCENTUAL FIXO DE IMPOSTO UTILIZADO NO MOMENTO DA APURAO ESTE CAMPO SERVE COMO HISTRICO CASO O USURIO
   ALTERE O PERCENTUAL FIXO DO IMPOSTO. */
               INSERT INTO ITRELACITROP
                  (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                   CDITRELITROP, QTITRELITROP, VRITRELITROP)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                   C_IMPOSTO.CDIMPOSTO, C_IMPOSTO.VRPERFIXCALCIMP, C_IMPOSTO.VRIMPOSTO);
            END LOOP;
         ELSE
            IMPOSTOROP_RELGESDIRETA(P_CDFILIAL, V_DTINIC, P_DTFINAL, V_NRAPURREL,
                                 V_NRROPFILIAL,  V_NRLANCROP, V_NRITLANCROP, C_EFETIVOS.CDCLIENTEAPGD,
                                 C_EFETIVOS.DTAPURACFATGD, C_EFETIVOS.CDSERVFILIAPGD);
         END IF;
      END LOOP;



      FOR C_EFE_PLA IN ( SELECT
                             CA.CDFILIAL, CA.DTCARDAPIO,
                             CA.CDSERVFILI,
                             SUM(CA.QTCOMEFETCAR) AS QTCOMEFETCAR
                            FROM
                              CARDADIA CA
                            WHERE
                              (CA.CDFILIAL   = P_CDFILIAL)   AND
                              (CA.DTCARDAPIO BETWEEN V_DTALTEFETIVO  AND P_DTFINAL) AND
                              (QTCOMEFETCAR >0)AND
                              (V_IDSERVFATROP   = 'S')
                            GROUP BY
                               CA.CDFILIAL, CA.DTCARDAPIO,  CA.CDSERVFILI

                            UNION

                             SELECT
                              CA.CDFILIAL, CA.DTCARDAPIO,
                              CA.CDSERVFILI,
                              SUM(CA.QTCOMEFETCAR) AS QTCOMEFETCAR
                            FROM
                               CARDADIA CA,
                               SERVICOROP S
                            WHERE
                               (CA.CDFILIAL   = P_CDFILIAL)   AND
                               (CA.DTCARDAPIO BETWEEN V_DTALTEFETIVO  AND P_DTFINAL)AND
                               (V_FILIALPAR       = S.CDFILIAL)                               AND
                               (CA.CDSERVFILI = S.CDSERVFILI)                             AND
                               (S.IDTPSERVICOROP   = 'E')                                      AND
                               (V_IDSERVFATROP   <> 'S') AND
                               (QTCOMEFETCAR >0)
                             GROUP BY
                               CA.CDFILIAL, CA.DTCARDAPIO,  CA.CDSERVFILI ) LOOP

         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_EFE_PLA.DTCARDAPIO THEN
            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
              INTO V_NRROPFILIAL
              FROM ROPFILIAL
             WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_EFE_PLA.DTCARDAPIO;


            V_DTLANCROP   := C_EFE_PLA.DTCARDAPIO;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;
           /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

        /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
           (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
            NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
         VALUES
           (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_EFE_PLA.DTCARDAPIO, 'EP', '',
            '',  '', '', '', '', V_CDFILCENTPME);

         V_NRITLANCROP := '00000';

        /*Inclui Item do Lanamento*/
         V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

         INSERT INTO ITLANCROPFILIAL
           (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
            NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP)
         VALUES
           (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, P_CDFILIAL, '',
            '', C_EFE_PLA.CDSERVFILI,  C_EFE_PLA.QTCOMEFETCAR, 0);
      END LOOP;

      DELETE FROM TRAB_APURAC_GD
      WHERE
        (CDFILIALAPGD   = P_CDFILIAL)                               AND
        (NRAPURACGD     = V_NRAPURREL)                              AND
        (DTAPURACFATGD  BETWEEN V_DTALTEFETIVO AND P_DTFINAL) ;

      UPDATE LOGALTEFET
         SET IDATUALROP = 'S'
       WHERE
          CDFILIAL =  P_CDFILIAL AND
          DTALTEFETIVO >= V_DTALTEFETIVO;
   END IF;

/* Faturamentos Diversos*/
   IF V_IDCONSFATUDIV = 'S' THEN
      V_NRITLANCROP := '00000';
      V_DTLANCROP := '01/01/1990';

      FOR C_FATUDIV IN ( SELECT L.CDFILIAL, L.CDFATUDIVER, L.DTLANCFATDIV, L.VRLANCFATDIV
                           FROM LANCFATUDIVE L
                          WHERE (L.CDFILIAL = P_CDFILIAL)  AND
                                (L.DTLANCFATDIV BETWEEN V_DTINIC -32 AND P_DTFINAL)
                         ORDER BY L.DTLANCFATDIV, L.CDFATUDIVER )LOOP

  /* Verifica se a faturamento j existe na tabela*/
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM LANCROPFILIAL L, ITLANCROPFILIAL I
             WHERE
               L.CDFILIAL =  C_FATUDIV.CDFILIAL AND
               L.DTLANCROP = C_FATUDIV.DTLANCFATDIV AND
               CDITEMLANCROP = C_FATUDIV.CDFATUDIVER AND
               L.CDFILIAL =  I.CDFILIAL AND
               L.NRROPFILIAL = I.NRROPFILIAL AND
               L.NRLANCROP = I.NRLANCROP AND
               IDTPLANCROP = 'FD';
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
            END;

         IF V_COUNT = 0 THEN

            V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
            IF V_DTLANCROP <> C_FATUDIV.DTLANCFATDIV THEN

               V_NRROPFILIAL := NULL;
               SELECT MAX(NRROPFILIAL)
               INTO V_NRROPFILIAL
               FROM ROPFILIAL
               WHERE CDFILIAL    = P_CDFILIAL AND
                     DTFILIALROP = C_FATUDIV.DTLANCFATDIV;


               V_DTLANCROP   := C_FATUDIV.DTLANCFATDIV;
               IF V_NRROPFILIAL IS NULL THEN
                  V_NRROPFILIAL := V_NRROPFILINI;
               END IF;

            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
               SELECT MAX(NRLANCROP)
                 INTO V_NRLANCROP
                 FROM LANCROPFILIAL
                WHERE CDFILIAL = P_CDFILIAL AND
                      NRROPFILIAL = V_NRROPFILIAL;

               IF V_NRLANCROP IS NULL THEN
                  V_NRLANCROP  := 0;
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               ELSE
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               END IF;
            END IF;

            INSERT INTO LANCROPFILIAL
               (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP,CDFILCENTROP)
            VALUES
               (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_FATUDIV.DTLANCFATDIV, 'FD', '', '',
               '', '', C_FATUDIV.CDFILIAL, '', V_CDFILCENTPME);

            V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

            INSERT INTO ITLANCROPFILIAL
               (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP, VRBRUTITLANCROP)
            VALUES
               (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_FATUDIV.CDFILIAL, '',
                '', C_FATUDIV.CDFATUDIVER,  1, C_FATUDIV.VRLANCFATDIV, 0);
         END IF;
      END LOOP;
   END IF;


/* Clculo do Consumo*/

   /*Consumo Compra*/
   IF V_IDCOMPRAROP = 'S' THEN
      V_DTLANCROP := '01/01/1990';
      FOR C_CONSCOMPRA IN (SELECT N.CDTIPOOPER, N.NRNOTAFISC, N.CDFILIAL, CDFILIRECE,
                                  N.NRLANCTONF, DTENTRSAID,   N.CDSERIE, N.CDFORNECED,
                                  N.IDSTATUS,   N.DTINIREFEFAT, N.DTFINREFEFAT, P.NUMERORE,
                                  /*L.NRLANCESTQ,*/ N.NRSEQCAPADOC
                             FROM NOTAFISCAL N, /*LANCTOESTOQ L,*/ TIPOOPERACROP PR,
                                 (SELECT NRNUMERNF AS NUMERORE, CDFILIAL, DTNUMERNF AS DATARE
                                    FROM NUMERORNF
                                   WHERE CDFILIAL = P_CDFILIAL
                                     AND IDENTRSAID = 'E'
                                     AND DTNUMERNF BETWEEN V_DTINIC AND P_DTFINAL) P
                            WHERE (N.CDFILIRECE    = P_CDFILIAL)
                              AND (N.IDENTRSAIDOP  = 'E')
                              AND (N.DTENTRSAID BETWEEN V_DTINIC  AND P_DTFINAL)
                              /*AND (N.CDFILIAL      = L.CDFILIAL)
                              AND (N.NRLANCTONF    = L.NRLANCTONF) */
                              AND (N.CDFILIAL      = P.CDFILIAL(+))
                              AND (N.DTENTRSAID    = P.DATARE(+))
                              AND (N.NRSEQCAPADOC = P.NUMERORE(+))
                              AND (N.IDSTATUS    <> 'C')
                              AND (N.IDSISTEMA NOT IN ('PAG','REC','FIS','CMP') OR N.IDSISTEMA IS NULL)
                              AND (V_FILIALPAR     = PR.CDFILIAL)
                              AND (N.CDTIPOOPER    = PR.CDTIPOOPER)
                              AND (N.IDENTRSAIDOP  = PR.IDENTRSAIDOP)
                              AND (PR.IDTPOPERACROP = 'CC')
                            ORDER BY DTENTRSAID)LOOP

         V_NRLANCROP := LPAD(V_NRLANCROP + 1,10,'0');

         IF V_DTLANCROP <> C_CONSCOMPRA.DTENTRSAID THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_CONSCOMPRA.DTENTRSAID;

            V_DTLANCROP   := C_CONSCOMPRA.DTENTRSAID;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;

            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

         /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP, IDSTATUSROP,
             DTINIREFEROP,  DTFINREFEROP, NUMEROREROP)
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_CONSCOMPRA.DTENTRSAID, 'CC', C_CONSCOMPRA.CDFORNECED,
             C_CONSCOMPRA.CDTIPOOPER,   C_CONSCOMPRA.NRNOTAFISC, C_CONSCOMPRA.CDSERIE, C_CONSCOMPRA.CDFILIAL,
             C_CONSCOMPRA.NRLANCTONF,   V_CDFILCENTPME, C_CONSCOMPRA.IDSTATUS, C_CONSCOMPRA.DTINIREFEFAT,
             C_CONSCOMPRA.DTFINREFEFAT, NVL(C_CONSCOMPRA.NRSEQCAPADOC, C_CONSCOMPRA.NUMERORE));

         V_NRITLANCROP   := '00000';
         V_NRSEQRELACIT  := 0;
         V_PRODUTO       := '';
         V_NRSEQIT       := 'N';
         V_PRIIMP        := 0;
/* VERIFICAR SE A QUANTIDADE DEVE SER CONVERTIDA*/
         FOR C_ITCONSCOMPRA IN(SELECT
                                    AN.CDSUBGRPROD, AN.CDGRUPPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                    SUM(DECODE(C_CONSCOMPRA.CDTIPOOPER, '03',I.VRBRUTITEM/H.VRFATOCONV, I.VRBRUTITEM)) AS VALORTOTAL,
                                    SUM(I.QTITEMPROD) AS QUANTIDADE
                                  FROM
                                     ITEMPRODUTO I, SUBGRPRODFIL SG, PRSUBGRPROD AN, PRODFILI PR, PRODUTO H
                                  WHERE
                                     (I.CDFILIAL      = C_CONSCOMPRA.CDFILIAL)      AND
                                     (I.NRLANCTONF    = C_CONSCOMPRA.NRLANCTONF)    AND
                                     (SG.CDFILISUBGR  = V_CDFILISUBGR)              AND
                                     (SG.CDGRUPPROD   = V_CDGRUPPROD )              AND
                                     (SG.CDFILISUBGR  = AN.CDFILISUBGR)             AND
                                     (SG.CDSUBGRPROD  = AN.CDSUBGRPROD)             AND
                                     (SG.CDGRUPPROD   = AN.CDGRUPPROD)              AND
                                     (I.CDPRODUTO     = AN.CDPRODUTO)               AND
                                     (H.CDPRODUTO = I.CDPRODUTO     )               AND
                                     (PR.CDFILIAL     = P_CDFILIAL)                 AND
                                     (PR.CDPRODUTO    = H.CDPRODESTO)               AND
                                     (NVL(PR.IDCNTRESTOQ,'S')  = 'S')
                                  GROUP BY AN.CDSUBGRPROD, AN.CDGRUPPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                                  ORDER BY AN.CDSUBGRPROD, AN.CDGRUPPROD)LOOP


               IF (NVL(V_PRODUTO,'N') <> C_ITCONSCOMPRA.CDSUBGRPROD)THEN
                  /*
                  BEGIN

                     SELECT SUM(VRCUSTLOGLAN) VRCUSTLOGLAN,
                            SUM(VRSEGULOGLAN) VRSEGULOGLAN
                            INTO V_VRCUSTLOGLAN, V_VRSEGULOGLAN
                       FROM GRUPPROD GR, SUBGRPRODFIL SG, PARFILIALROP P,
                            PRSUBGRPROD AN, LANCTOESTOQ L,
                            ITLANCTOEST IT, PRODFILI PR
                       WHERE
                          (IT.CDFILIAL     = C_CONSCOMPRA.CDFILIAL)      AND
                          (IT.NRLANCESTQ   = C_CONSCOMPRA.NRLANCESTQ)    AND
                          (P.CDFILIAL      = V_FILIALPAR)                AND
                          (SG.CDFILISUBGR  = V_CDFILISUBGR)              AND
                          (AN.CDSUBGRPROD  = C_ITCONSCOMPRA.CDSUBGRPROD) AND
                          (P.CDGRUPPROD    = GR.CDGRUPPROD)              AND
                          (GR.CDGRUPPROD   = SG.CDGRUPPROD)              AND
                          (SG.CDSUBGRPROD  = AN.CDSUBGRPROD)             AND
                          (IT.CDPRODUTO    = AN.CDPRODUTO)               AND
                          (SG.CDGRUPPROD   = AN.CDGRUPPROD)              AND
                          (L.NRLANCTONF    = C_CONSCOMPRA.NRLANCTONF)    AND
                          (L.CDFILIAL      = IT.CDFILIAL)                AND
                          (L.NRLANCESTQ    = IT.NRLANCESTQ)              AND
                          (PR.CDFILIAL     = P_CDFILIAL)                 AND
                          (PR.CDPRODUTO    = IT.CDPRODMOVI)              AND
                          (NVL(PR.IDCNTRESTOQ,'S') = 'S')                AND
                          (NVL(IT.IDEVOLESTOQ, 'S') = 'S');
                  EXCEPTION
                     WHEN NO_DATA_FOUND THEN BEGIN
                        V_VRCUSTLOGLAN   := 0;
                        V_VRSEGULOGLAN   := 0;
                     END;
                  END; */
                  V_VRCUSTLOGLAN   := 0;
                  V_VRSEGULOGLAN   := 0;


                  /*Inclui Item do Lanamento*/
                  V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');
                  V_PRODUTO     := C_ITCONSCOMPRA.CDSUBGRPROD;

                  INSERT INTO ITLANCROPFILIAL
                     (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                      NRSEQITPRODROP, CDITEMLANCROP  ,QTITEMLANCROP, VRITEMLANCROP,
                      CDGRUPPRODROP, CDSUBGRPRODROP, VRCUSTOLOGITROP, VRCUSTSEGPROD, IDTPSUBGRUPPROD)
                  VALUES
                    (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_CONSCOMPRA.CDFILIAL, C_CONSCOMPRA.NRLANCTONF,
                     '', C_ITCONSCOMPRA.CDSUBGRPROD, C_ITCONSCOMPRA.QUANTIDADE, C_ITCONSCOMPRA.VALORTOTAL,
                     C_ITCONSCOMPRA.CDGRUPPROD, C_ITCONSCOMPRA.CDSUBGRPROD, V_VRCUSTLOGLAN,V_VRSEGULOGLAN, C_ITCONSCOMPRA.IDSUBGRMATPRIMA );

                  V_CONSUMO := V_CONSUMO + C_ITCONSCOMPRA.VALORTOTAL;
               END IF;
            IF V_PRIIMP = 0 THEN
               FOR C_IMPOSTOCC IN
                  (SELECT R.CDIMPOSTO, SUM(VRIMPOSTOIT) AS VALORIMPOSTO
                     FROM IMPOSTOROP R, ITEMPROIMPOS T, ITEMPRODUTO I
                    WHERE (I.CDFILIAL     = C_CONSCOMPRA.CDFILIAL)
                      AND (I.NRLANCTONF   = C_CONSCOMPRA.NRLANCTONF)
                      AND (I.CDFILIAL     = T.CDFILIAL)
                      AND (I.NRLANCTONF   = T.NRLANCTONF)
                      AND (I.NRSEQITPROD  = T.NRSEQITPROD)
                      AND (T.NRITPROIMPOS = '1')
                      AND (T.CDIMPOSTO    = R.CDIMPOSTO)
                      AND (R.CDFILIAL     = V_FILIALPAR)
                      AND (R.IDTPIMPOSTOROP = 'CC')
                    GROUP BY R.CDIMPOSTO) LOOP

                    V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

                  INSERT INTO ITRELACITROP
                     (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                      CDITRELITROP, QTITRELITROP, VRITRELITROP)
                  VALUES
                     (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                      C_IMPOSTOCC.CDIMPOSTO, 0, C_IMPOSTOCC.VALORIMPOSTO);

                  V_PRIIMP := 1;
               END LOOP;
            END IF;

         END LOOP;


      END LOOP;
   END IF;

   /*Consumo Sada*/

   IF V_IDNFSAIDAROP = 'S' THEN
      V_DTLANCROP := '01/01/1990';
      FOR C_CONSSAIDA IN (SELECT N.CDCLIENTE,  N.CDTIPOOPER, N.NRNOTAFISC, N.CDFILIAL,
                                 N.NRLANCTONF, DTENTRSAID,   N.CDSERIE, N.CDFORNECED,
                                 N.IDSTATUS,   N.DTINIREFEFAT, N.DTFINREFEFAT, P.NUMERORE, L.NRLANCESTQ
                            FROM NOTAFISCAL N, LANCTOESTOQ L, TIPOOPERACROP PR,
                                 (SELECT NRNUMERNF AS NUMERORE, CDFILIAL, DTNUMERNF AS DATARE
                                    FROM NUMERORNF
                                   WHERE CDFILIAL = P_CDFILIAL
                                     AND IDENTRSAID = 'S'
                                     AND DTNUMERNF BETWEEN V_DTINIC AND P_DTFINAL) P
                          WHERE (N.CDFILIEMIT   = P_CDFILIAL)
                            AND (N.IDENTRSAIDOP = 'S')
                            AND (N.DTENTRSAID BETWEEN V_DTINIC  AND P_DTFINAL)
                            AND (N.CDFILIAL   = L.CDFILIAL)
                            AND (N.NRLANCTONF = L.NRLANCTONF)
                            AND (N.CDFILIAL   = P.CDFILIAL(+))
                            AND (N.DTENTRSAID = P.DATARE(+))
                            AND (N.NRSEQCAPADOC = P.NUMERORE(+))
                            AND (N.IDSTATUS    <> 'C')
                            AND (V_FILIALPAR     = PR.CDFILIAL)
                            AND (N.CDTIPOOPER    = PR.CDTIPOOPER)
                            AND (N.IDENTRSAIDOP  = PR.IDENTRSAIDOP)
                            AND (PR.IDTPOPERACROP = 'CS')
                            AND (N.IDSISTEMA NOT IN ('PAG','REC','FIS','CMP')  OR N.IDSISTEMA IS NULL))LOOP

         V_NRLANCROP := LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_CONSSAIDA.DTENTRSAID THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_CONSSAIDA.DTENTRSAID;


            V_DTLANCROP   := C_CONSSAIDA.DTENTRSAID;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;

            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;

         END IF;

         /*Inclui Lanamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP, IDSTATUSROP,
             DTINIREFEROP,  DTFINREFEROP, NUMEROREROP)
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_CONSSAIDA.DTENTRSAID, 'CS', C_CONSSAIDA.CDFORNECED,
             C_CONSSAIDA.CDTIPOOPER,  C_CONSSAIDA.NRNOTAFISC, C_CONSSAIDA.CDSERIE, C_CONSSAIDA.CDFILIAL,
             C_CONSSAIDA.NRLANCTONF, V_CDFILCENTPME, C_CONSSAIDA.IDSTATUS, C_CONSSAIDA.DTINIREFEFAT,
             C_CONSSAIDA.DTFINREFEFAT, C_CONSSAIDA.NUMERORE);

         V_NRITLANCROP  := '00000';
         V_NRSEQRELACIT := 0;
         V_PRODUTO      := 'N';
         V_NRSEQIT      := 'N';


         SELECT MAX(CDFILIALIT), MAX(NRLANCTONFIT)
         INTO V_CDFILIALIT, V_NRLANCTONFIT
         FROM ITEMNOTAFISC
         WHERE CDFILIAL = C_CONSSAIDA.CDFILIAL
         AND NRLANCTONF = C_CONSSAIDA.NRLANCTONF;

         V_PRIIMP := 0;
         FOR C_ITCONSSAIDA IN(SELECT
                                 AN.CDSUBGRPROD, AN.CDGRUPPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                 SUM(I.VRBRUTITEM) AS VALORTOTAL,
                                 SUM(I.QTITEMPROD) AS QUANTIDADE
                              FROM
                               ITEMPRODUTO I,
                               SUBGRPRODFIL SG, PRSUBGRPROD AN
                              WHERE
                                 (I.CDFILIAL     = C_CONSSAIDA.CDFILIAL)       AND
                                 (I.NRLANCTONF   = C_CONSSAIDA.NRLANCTONF)     AND
                                 (SG.CDFILISUBGR = V_CDFILISUBGR)              AND
                                 (SG.CDGRUPPROD  = V_CDGRUPPROD )              AND
                                 (AN.CDFILISUBGR = SG.CDFILISUBGR)             AND
                                 (AN.CDGRUPPROD  = SG.CDGRUPPROD)              AND
                                 (AN.CDSUBGRPROD = SG.CDSUBGRPROD)             AND
                                 (I.CDPRODMOVI   = AN.CDPRODUTO)               AND
                                 (I.CDPRODMOVI   = (SELECT DISTINCT IT.CDPRODMOVI
                                                       FROM ITLANCTOEST IT
                                                      WHERE IT.CDFILIAL   = C_CONSSAIDA.CDFILIAL
                                                        AND IT.NRLANCESTQ = C_CONSSAIDA.NRLANCESTQ
                                                        AND I.CDPRODMOVI = IT.CDPRODMOVI
                                                        AND NVL(IDEVOLESTOQ, 'S') = 'S'))
                              GROUP BY AN.CDSUBGRPROD, AN.CDGRUPPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                              ORDER BY AN.CDSUBGRPROD, AN.CDGRUPPROD)LOOP

            IF (NVL(V_PRODUTO,'N') <> C_ITCONSSAIDA.CDSUBGRPROD)THEN

               /*Inclui Item do Lanamento*/
               V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');
               V_PRODUTO := C_ITCONSSAIDA.CDSUBGRPROD;

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP, QTITEMLANCROP, VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_CONSSAIDA.CDFILIAL, C_CONSSAIDA.NRLANCTONF,
                   '', C_ITCONSSAIDA.CDSUBGRPROD,  C_ITCONSSAIDA.QUANTIDADE, C_ITCONSSAIDA.VALORTOTAL,
                   C_ITCONSSAIDA.CDGRUPPROD, C_ITCONSSAIDA.CDSUBGRPROD, C_ITCONSSAIDA.IDSUBGRMATPRIMA);

               V_CONSUMO := V_CONSUMO - C_ITCONSSAIDA.VALORTOTAL;

            END IF;

            IF V_PRIIMP = 0 THEN
               FOR C_IMPOSTOCS IN
                  (SELECT R.CDIMPOSTO, SUM(VRIMPOSTOIT) AS VALORIMPOSTO
                     FROM IMPOSTOROP R, ITEMPROIMPOS T, ITEMPRODUTO I
                    WHERE (I.CDFILIAL     = C_CONSSAIDA.CDFILIAL)
                      AND (I.NRLANCTONF   = C_CONSSAIDA.NRLANCTONF)
                      AND (I.CDFILIAL     = T.CDFILIAL)
                      AND (I.NRLANCTONF   = T.NRLANCTONF)
                      AND (I.NRSEQITPROD  = T.NRSEQITPROD)
                      AND (T.NRITPROIMPOS = '1')
                      AND (T.CDIMPOSTO    = R.CDIMPOSTO)
                      AND (R.CDFILIAL     = V_FILIALPAR)
                      AND (R.IDTPIMPOSTOROP = 'CS')
                    GROUP BY R.CDIMPOSTO) LOOP

                    V_NRSEQRELACIT := V_NRSEQRELACIT + 1;

                  INSERT INTO ITRELACITROP
                     (CDFILIAL,  NRROPFILIAL, NRLANCROP, NRITLANCROP, NRSEQRELACITROP,
                      CDITRELITROP, QTITRELITROP, VRITRELITROP)
                  VALUES
                     (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, V_NRSEQRELACIT,
                      C_IMPOSTOCS.CDIMPOSTO, 0, C_IMPOSTOCS.VALORIMPOSTO);

                  V_PRIIMP := 1;
               END LOOP;
            END IF;
         END LOOP;
      END LOOP;
   END IF;

   IF (V_IDIMPLANTROP = 'S' OR V_IDAJUSTEROP = 'S') THEN
      V_DTLANCROP := '01/01/1990';

      SELECT MAX(CDARVPROD) AS CDPRODFIN,
             MIN(CDARVPROD) AS CDPRODINI
        INTO V_CDPRODFIN, V_CDPRODINI
        FROM PRODUTO P, ITLANCTOEST I, PRSUBGRPROD AN
       WHERE CDFILIMOVI  =  P_CDFILIAL AND
             DTLANCMOVI BETWEEN V_DTINIC  AND P_DTFINAL AND
             I.CDPRODMOVI = P.CDPRODUTO                 AND
             IDTIPOMOVI IN ('0','1') AND
             AN.CDGRUPPROD  = V_CDGRUPPROD  AND
             AN.CDFILISUBGR = V_FILIALPAR AND
             AN.CDPRODUTO = I.CDPRODMOVI;

      IF V_CDPRODFIN IS NOT NULL THEN
         SELECT LPAD(ABS(ROUND(DBMS_RANDOM.VALUE(9999,9999999999))),10,'0')
         INTO V_NRMOVIESTQ
         FROM DUAL;

         MOVIMENTACAO_ESTOQUE(P_CDFILIAL,NULL,NULL,NULL, 1, V_DTINIC, P_DTFINAL,  V_CDPRODINI, V_CDPRODFIN, 3, 1,V_NRMOVIESTQ, 'S');

         V_PRODUTO := '';
         V_NRSEQIT := 'N';                         /* ALTERADO PARA GRAVAR POR SUBGRUPO */
         IF V_IDIMPLANTROP = 'S' THEN
            V_DTLANCROP := '01/01/1990';

            FOR C_IMPLANTACAO IN (SELECT AN.CDGRUPPROD, AN.CDSUBGRPROD, DTLANCMOVIEST, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                         SUM(QTLANCAMENTO) QTLANCAMENTO,
                                         SUM(VRDIFECUST) VRDIFECUST
                                    FROM RELAT_MOVIEST R , PRODUTO P,  SUBGRPRODFIL SG,  PRSUBGRPROD AN
                                   WHERE (R.CDFILIMOVIEST = P_CDFILIAL)
                                     AND (R.NRRELATMOVI   = V_NRMOVIESTQ)
                                     AND (R.DTLANCMOVIEST BETWEEN V_DTINIC  AND P_DTFINAL)
                                     AND (SG.CDFILISUBGR  = V_CDFILISUBGR)
                                    AND  SG.CDGRUPPROD =  V_CDGRUPPROD
                                    AND  AN.CDFILISUBGR = SG.CDFILISUBGR
                                    AND  AN.CDGRUPPROD = SG.CDGRUPPROD
                                    AND  AN.CDSUBGRPROD = SG.CDSUBGRPROD
                                    AND (P.CDPRODUTO     = AN.CDPRODUTO)
                                    AND (R.IDTIPOMOVIEST = '0')
                                    AND (R.CDPRODMOVIEST   = P.CDARVPROD)
                                   GROUP BY DTLANCMOVIEST, AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                                   ORDER BY DTLANCMOVIEST, AN.CDSUBGRPROD, AN.CDGRUPPROD) LOOP


               IF V_DTLANCROP <> C_IMPLANTACAO.DTLANCMOVIEST THEN

                  V_NRROPFILIAL := NULL;
                  SELECT MAX(NRROPFILIAL)
                  INTO V_NRROPFILIAL
                  FROM ROPFILIAL
                  WHERE CDFILIAL    = P_CDFILIAL AND
                        DTFILIALROP = C_IMPLANTACAO.DTLANCMOVIEST;


                  V_DTLANCROP   := C_IMPLANTACAO.DTLANCMOVIEST;
                  IF V_NRROPFILIAL IS NULL THEN
                     V_NRROPFILIAL := V_NRROPFILINI;
                  END IF;

                  /* PEGA O ULTIMO NUMERO DO LANAMENTO */
                  SELECT MAX(NRLANCROP)
                    INTO V_NRLANCROP
                    FROM LANCROPFILIAL
                   WHERE CDFILIAL = P_CDFILIAL AND
                         NRROPFILIAL = V_NRROPFILIAL;

                  IF V_NRLANCROP IS NULL THEN
                     V_NRLANCROP  := 0;
                     V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
                  ELSE
                     V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
                  END IF;
               END IF;

                /*Inclui implantao*/
               V_NRLANCROP := LPAD(V_NRLANCROP + 1,10,'0');

               INSERT INTO LANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                   NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
               VALUES
                  (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_IMPLANTACAO.DTLANCMOVIEST, 'IM', '',
                   '',  '', '', '','', V_CDFILCENTPME);

               /*Inclui Item do Lanamento*/
               V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP, VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, '', '',
                   '', C_IMPLANTACAO.CDSUBGRPROD,  C_IMPLANTACAO.QTLANCAMENTO, C_IMPLANTACAO.VRDIFECUST,
                   C_IMPLANTACAO.CDGRUPPROD, C_IMPLANTACAO.CDSUBGRPROD, C_IMPLANTACAO.IDSUBGRMATPRIMA);

               V_CONSUMO := V_CONSUMO + C_IMPLANTACAO.VRDIFECUST;
            END LOOP;
         END IF;

         V_PRODUTO := '';
         V_NRSEQIT := 'N';

         IF V_IDAJUSTEROP = 'S' THEN
            V_DTLANCROP := '01/01/1990';
            FOR C_AJUSTE IN (SELECT AN.CDGRUPPROD, AN.CDSUBGRPROD, DTLANCMOVIEST, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                    SUM(QTLANCAMENTO) QTLANCAMENTO, SUM(VRDIFECUST) VRDIFECUST
                             FROM RELAT_MOVIEST R, PRODUTO  P, SUBGRPRODFIL SG,  PRSUBGRPROD AN
                             WHERE (R.CDFILIMOVIEST = P_CDFILIAL)
                               AND (R.NRRELATMOVI   = V_NRMOVIESTQ)
                               AND (R.DTLANCMOVIEST BETWEEN V_DTINIC  AND P_DTFINAL)
                               AND (SG.CDGRUPPROD = V_CDGRUPPROD)
                               AND (SG.CDFILISUBGR  = V_CDFILISUBGR)
                               AND AN.CDFILISUBGR = SG.CDFILISUBGR
                               AND AN.CDGRUPPROD  = SG.CDGRUPPROD
                               AND AN.CDSUBGRPROD = SG.CDSUBGRPROD
                               AND (P.CDPRODUTO     = AN.CDPRODUTO)
                               AND (R.IDTIPOMOVIEST = '1')
                               AND (R.CDPRODMOVIEST   = P.CDARVPROD)
                             GROUP BY DTLANCMOVIEST, AN.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')
                             ORDER BY DTLANCMOVIEST, AN.CDSUBGRPROD, AN.CDGRUPPROD) LOOP


/*Inclui ajuste*/
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');

               IF V_DTLANCROP <> C_AJUSTE.DTLANCMOVIEST THEN

                  V_NRROPFILIAL := NULL;
                  SELECT MAX(NRROPFILIAL)
                  INTO V_NRROPFILIAL
                  FROM ROPFILIAL
                  WHERE CDFILIAL    = P_CDFILIAL AND
                        DTFILIALROP = C_AJUSTE.DTLANCMOVIEST;


                  V_DTLANCROP   := C_AJUSTE.DTLANCMOVIEST;
                  IF V_NRROPFILIAL IS NULL THEN
                     V_NRROPFILIAL := V_NRROPFILINI;
                  END IF;

/* PEGA O ULTIMO NUMERO DO LANAMENTO */
                  SELECT MAX(NRLANCROP)
                    INTO V_NRLANCROP
                    FROM LANCROPFILIAL
                   WHERE CDFILIAL = P_CDFILIAL AND
                         NRROPFILIAL = V_NRROPFILIAL;

                  IF V_NRLANCROP IS NULL THEN
                     V_NRLANCROP  := 0;
                     V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
                  ELSE
                     V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
                  END IF;
               END IF;

               INSERT INTO LANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                   NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
               VALUES
                  (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_AJUSTE.DTLANCMOVIEST, 'AJ', '',
                   '',  '', '', '', '', V_CDFILCENTPME);

               /*Inclui Item do Lanamento*/

               V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP, VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD )
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, '',
                   '', '', C_AJUSTE.CDSUBGRPROD,  C_AJUSTE.QTLANCAMENTO, C_AJUSTE.VRDIFECUST,
                   C_AJUSTE.CDGRUPPROD, C_AJUSTE.CDSUBGRPROD, C_AJUSTE.IDSUBGRMATPRIMA);
            END LOOP;
         END IF;
      END IF;
   END IF;

/*Clculo da Retirada*/

   /*Retirada de Estoque*/
   IF V_IDRETIRADAROP = 'E' OR V_IDRETIRADAROP = 'A' THEN
      V_DTLANCROP := '01/01/1990';
      FOR C_RETIRADA IN (SELECT L.CDFILIAL, L.NRLANCESTQ, L.DTLANCESTQ
                         FROM  RETIRADAESTO R, LANCTOESTOQ L
                         WHERE (L.CDFILIMOVI = P_CDFILIAL  )                              AND
                               (L.DTLANCESTQ BETWEEN V_DTINIC  AND P_DTFINAL)          AND
                               (L.IDTPLANCTO = '3'          )                             AND
                               (L.CDFILIAL   = R.CDFILIALLAN)                             AND
                               (L.NRLANCESTQ = R.NRLANCESTQ )
                         ORDER BY L.NRLANCESTQ,   L.DTLANCESTQ) LOOP

         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
          /*Inclui Retirada de Estoque*/
         IF V_DTLANCROP <> C_RETIRADA.DTLANCESTQ THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_RETIRADA.DTLANCESTQ;


            V_DTLANCROP   := C_RETIRADA.DTLANCESTQ;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;
            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP )
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_RETIRADA.DTLANCESTQ, 'RE', '',
             '',  '', '', C_RETIRADA.CDFILIAL, C_RETIRADA.NRLANCESTQ, V_CDFILCENTPME);

            /* verificar se as retiradas de estoque  sofreram alguma alteracao QUANTO AO AGRUPAMENTO*/

            FOR C_ITRETIRADA IN(SELECT CDPRODMOVI, IT.QTLANCTOEST, SG.CDGRUPPROD, AN.CDSUBGRPROD,
                                (CUSTO_PRODUTO_LCEST(IT.CDFILIMOVI, IT.CDPRODMOVI,
                                                     V_NRTIPOCUSTO, C_RETIRADA.DTLANCESTQ,
                                                     SYSDATE, CDALMOXARIFE, CDLOCALESTOQ,
                                                     NRLOTEESTQ, NRSUBLOTE) * IT.QTLANCTOEST) as VRLANCTOESTQ
                                  FROM ITLANCTOEST IT, SUBGRPRODFIL SG, PRSUBGRPROD AN
                                 WHERE (IT.CDFILIAL   = C_RETIRADA.CDFILIAL  )
                                   AND (IT.NRLANCESTQ = C_RETIRADA.NRLANCESTQ)
                                   AND (NVL(IT.IDEVOLESTOQ, 'S') = 'S')
                                   AND SG.CDGRUPPROD  = V_CDGRUPPROD
                                   AND SG.CDFILISUBGR = V_CDFILISUBGR
                                   AND SG.CDFILISUBGR = AN.CDFILISUBGR
                                   AND SG.CDGRUPPROD  = AN.CDGRUPPROD
                                   AND SG.CDSUBGRPROD = AN.CDSUBGRPROD
                                   AND IT.CDPRODMOVI  = AN.CDPRODUTO) LOOP

               /*Inclui Item d Retirada*/
               V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

               INSERT INTO ITLANCROPFILIAL
                  (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                   NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                   CDGRUPPRODROP, CDSUBGRPRODROP)
               VALUES
                  (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_RETIRADA.CDFILIAL, C_RETIRADA.NRLANCESTQ,
                   '', C_ITRETIRADA.CDPRODMOVI,  C_ITRETIRADA.QTLANCTOEST, C_ITRETIRADA.VRLANCTOESTQ,
                   C_ITRETIRADA.CDGRUPPROD, C_ITRETIRADA.CDSUBGRPROD);
            END LOOP;
      END LOOP;
   END IF;


   /*Retirada por Planejamento*/

   IF (V_IDRETIRADAROP = 'P' OR V_IDRETIRADAROP = 'A') AND (V_IDSERVRETROP <> 'S') THEN
      V_DTLANCROP := '01/01/1990';
      FOR C_RETIRADAPLA IN (SELECT DISTINCT PRO.CDSERVFILI, PRO.DTCARDAPIO, PRO.CDFILIALLANC, PRO.NRLANCESTQ
                              FROM (SELECT C.CDFILIAL, C.CDSERVFILI, DTCARDAPIO
                                      FROM CARDADIA C, SERVICOROP P
                                     WHERE (C.CDFILIAL   = P_CDFILIAL)                 AND
                                           (P.CDFILIAL   = V_FILIALPAR)                AND
                                           (C.CDSERVFILI = P.CDSERVFILI)               AND
                                           (DTCARDAPIO BETWEEN V_DTINIC  AND P_DTFINAL)
                                     GROUP BY C.CDFILIAL, C.CDSERVFILI, DTCARDAPIO) COMENS,
                                    (SELECT CA.CDFILIAL, CA.CDSERVFILI, CA.DTCARDAPIO,
                                            IL.CDFILIAL AS CDFILIALLANC, IL.NRLANCESTQ
                                       FROM CARDADIA CA, ITEMRETI IR, ITLANCTOEST IL,
                                            FILIAL   FI, SERVICO  SE, SERVICOROP P, PRODUTO PRA
                                      WHERE (CA.CDFILIAL   = P_CDFILIAL )                        AND
                                            (P.CDFILIAL    = V_FILIALPAR)                        AND
                                            (CA.CDSERVFILI =  P.CDSERVFILI)                      AND
                                            (NOT IR.QTRETIRADA IS NULL AND IR.QTRETIRADA <> 0)   AND
                                            (IL.CDFILIAL    = IR.CDFIITLANEST)                   AND
                                            (IL.NRLANCESTQ  = IR.NRLANCESTQ)                     AND
                                            (IL.NRSEQUITEM  = IR.NRSEQUITEM)                     AND
                                            (NVL(IL.IDEVOLESTOQ,'S') = 'S')                               AND
                                            (CA.DTCARDAPIO BETWEEN V_DTINIC  AND P_DTFINAL)   AND
                                            (CA.CDFILIAL    = IR.CDFILIAL)                       AND
                                            (CA.NRCARDAPIO  = IR.NRCARDAPIO)                     AND
                                            (PRA.CDPRODUTO  = IR.CDPRODUTO)                      AND
                                            (FI.CDFILIAL    = CA.CDFILIAL)                       AND
                                            (SE.CDFILIAL    = CA.CDFILIAL)                       AND
                                            (SE.CDSERVFILI  = CA.CDSERVFILI)) PRO
                             WHERE (COMENS.CDFILIAL(+)   = PRO.CDFILIAL) AND
                                   (COMENS.CDSERVFILI(+) = PRO.CDSERVFILI) AND
                                   (COMENS.DTCARDAPIO(+) = PRO.DTCARDAPIO)) LOOP


         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_RETIRADAPLA.DTCARDAPIO THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_RETIRADAPLA.DTCARDAPIO;


            V_DTLANCROP   := C_RETIRADAPLA.DTCARDAPIO;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;

            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

          /*Inclui Retirada de Planejamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP)
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_RETIRADAPLA.DTCARDAPIO, 'RP', '',
             '',  '', '', C_RETIRADAPLA.CDFILIALLANC, C_RETIRADAPLA.NRLANCESTQ, V_CDFILCENTPME);

          V_NRITLANCROP := '00000';
          FOR C_ITRETIRADAPLA IN(   SELECT
                                       R.CDFILIAL, CDSERVFILI, DTCARDAPIO, SG.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                       SUM(QTRETIRADA) QTRETIRADA,
                                       SUM(VRCUSTPROD) AS VRCUSTPROD
                                     FROM SUBGRPRODFIL SG, PRSUBGRPROD AN,
                                       (SELECT
                                          CDFILIAL,
                                          CDPRODUTO , CDSERVFILI,
                                          DTCARDAPIO, QTCOMEFETCAR, QTCLIEFETCAR, QTRETIRADA,
                                          CUSTO_PRODUTO(CDFILIAL, CDPRODUTO, V_NRTIPOCUSTO, DTCARDAPIO, SYSDATE) * QTRETIRADA AS VRCUSTPROD
                                        FROM
                                          (SELECT
                                             CA.CDFILIAL, CA.CDSERVFILI, CA.DTCARDAPIO,
                                             CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
                                             CA.QTCOMEFETCAR, CA.QTCLIEFETCAR, IR.CDPRODUTO,
                                             SUM(IR.QTRETIRADA * PRA.VRFATOCONV) AS QTRETIRADA
                                          FROM
                                            CARDADIA CA, ITLANCTOEST IL,
                                            SERVICOROP P, ITEMRETI IR, PRODUTO PRA
                                          WHERE
                                             (CA.CDFILIAL    = P_CDFILIAL )                      AND
                                             (P.CDFILIAL     = V_FILIALPAR)                      AND
                                             (CA.CDSERVFILI  =  P.CDSERVFILI)                    AND
                                             (IDTPSERVICOROP = 'R')                              AND
                                             (IL.CDFILIAL    = C_RETIRADAPLA.CDFILIALLANC)       AND
                                             (IL.NRLANCESTQ  = C_RETIRADAPLA.NRLANCESTQ)         AND
                                             (IL.CDFILIAL    = IR.CDFIITLANEST)                  AND
                                             (IL.NRLANCESTQ  = IR.NRLANCESTQ)                    AND
                                             (IL.NRSEQUITEM  = IR.NRSEQUITEM)                    AND
                                             (NVL(IL.IDEVOLESTOQ,'S') = 'S')                              AND
                                             (CA.CDFILIAL   = IR.CDFILIAL)                       AND
                                             (CA.NRCARDAPIO = IR.NRCARDAPIO)                     AND
                                             (PRA.CDPRODUTO = IR.CDPRODUTO)
                                          GROUP BY
                                             CA.CDFILIAL,CA.CDSERVFILI,CA.DTCARDAPIO,CA.QTCOMEFETCAR,
                                             CA.QTCLIEFETCAR, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
                                             IR.CDPRODUTO)) R
                                    WHERE
                                          SG.CDFILISUBGR = V_CDFILISUBGR
                                      AND SG.CDGRUPPROD  = V_CDGRUPPROD
                                      AND R.CDPRODUTO    = AN.CDPRODUTO
                                      AND SG.CDFILISUBGR = AN.CDFILISUBGR
                                      AND SG.CDGRUPPROD  = AN.CDGRUPPROD
                                      AND SG.CDSUBGRPROD = AN.CDSUBGRPROD
                                   GROUP BY R.CDFILIAL, DTCARDAPIO, CDSERVFILI, SG.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')) LOOP

             /*Inclui Item d Retirada*/
             V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

             INSERT INTO ITLANCROPFILIAL
                (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                 NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                 CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
             VALUES
                (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_RETIRADAPLA.CDFILIALLANC, C_RETIRADAPLA.NRLANCESTQ,
                 '', C_ITRETIRADAPLA.CDSERVFILI,  C_ITRETIRADAPLA.QTRETIRADA, C_ITRETIRADAPLA.VRCUSTPROD,
                 C_ITRETIRADAPLA.CDGRUPPROD, C_ITRETIRADAPLA.CDSUBGRPROD, C_ITRETIRADAPLA.IDSUBGRMATPRIMA);
          END LOOP;
      END LOOP;
   END IF;

   IF (V_IDRETIRADAROP = 'P' OR V_IDRETIRADAROP = 'A') AND (V_IDSERVRETROP = 'S') THEN
      V_DTLANCROP := '01/01/1990';
      FOR C_RETIRADAPLA IN (SELECT DISTINCT PRO.CDSERVFILI, PRO.DTCARDAPIO, PRO.CDFILIALLANC, PRO.NRLANCESTQ
                              FROM
                                 (SELECT C.CDFILIAL, C.CDSERVFILI, DTCARDAPIO
                                    FROM CARDADIA C
                                   WHERE (C.CDFILIAL   = P_CDFILIAL)                 AND
                                         (DTCARDAPIO BETWEEN V_DTINIC  AND P_DTFINAL)
                                   GROUP BY C.CDFILIAL, C.CDSERVFILI, DTCARDAPIO) COMENS,
                                 (SELECT CA.CDFILIAL, CA.CDSERVFILI, CA.DTCARDAPIO,
                                         IL.CDFILIAL AS CDFILIALLANC, IL.NRLANCESTQ
                                    FROM CARDADIA CA, ITEMRETI IR, ITLANCTOEST IL,
                                         FILIAL   FI, SERVICO  SE, PRODUTO PRA
                                    WHERE (CA.CDFILIAL   = P_CDFILIAL )                        AND
                                          (NOT IR.QTRETIRADA IS NULL AND IR.QTRETIRADA <> 0)   AND
                                          (IL.CDFILIAL    = IR.CDFIITLANEST)                   AND
                                          (IL.NRLANCESTQ  = IR.NRLANCESTQ)                     AND
                                          (IL.NRSEQUITEM  = IR.NRSEQUITEM)                     AND
                                          (NVL(IL.IDEVOLESTOQ,'S') = 'S')                               AND
                                          (CA.DTCARDAPIO BETWEEN V_DTINIC  AND P_DTFINAL)   AND
                                          (CA.CDFILIAL    = IR.CDFILIAL)                       AND
                                          (CA.NRCARDAPIO  = IR.NRCARDAPIO)                     AND
                                          (PRA.CDPRODUTO  = IR.CDPRODUTO)                      AND
                                          (FI.CDFILIAL    = CA.CDFILIAL)                       AND
                                          (SE.CDFILIAL    = CA.CDFILIAL)                       AND
                                          (SE.CDSERVFILI  = CA.CDSERVFILI)) PRO
                        WHERE (COMENS.CDFILIAL(+)   = PRO.CDFILIAL) AND
                              (COMENS.CDSERVFILI(+) = PRO.CDSERVFILI) AND
                              (COMENS.DTCARDAPIO(+) = PRO.DTCARDAPIO)) LOOP

         V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
         IF V_DTLANCROP <> C_RETIRADAPLA.DTCARDAPIO THEN

            V_NRROPFILIAL := NULL;
            SELECT MAX(NRROPFILIAL)
            INTO V_NRROPFILIAL
            FROM ROPFILIAL
            WHERE CDFILIAL    = P_CDFILIAL AND
                  DTFILIALROP = C_RETIRADAPLA.DTCARDAPIO;


            V_DTLANCROP   := C_RETIRADAPLA.DTCARDAPIO;
            IF V_NRROPFILIAL IS NULL THEN
               V_NRROPFILIAL := V_NRROPFILINI;
            END IF;

/* PEGA O ULTIMO NUMERO DO LANAMENTO */
            SELECT MAX(NRLANCROP)
              INTO V_NRLANCROP
              FROM LANCROPFILIAL
             WHERE CDFILIAL = P_CDFILIAL AND
                   NRROPFILIAL = V_NRROPFILIAL;

            IF V_NRLANCROP IS NULL THEN
               V_NRLANCROP  := 0;
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            ELSE
               V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
            END IF;
         END IF;

          /*Inclui Retirada de Planejamento*/
         INSERT INTO LANCROPFILIAL
            (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
             NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP)
         VALUES
            (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_RETIRADAPLA.DTCARDAPIO, 'RP', '',
             '',  '', '', C_RETIRADAPLA.CDFILIALLANC, C_RETIRADAPLA.NRLANCESTQ, V_CDFILCENTPME);

          V_NRITLANCROP := '00000';

         FOR C_ITRETIRADAPLA IN(SELECT
                                    R.CDFILIAL, CDSERVFILI, SG.CDGRUPPROD, AN.CDSUBGRPROD, DTCARDAPIO, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P') AS IDSUBGRMATPRIMA,
                                    SUM(VRCUSTPROD) AS VRCUSTPROD,
                                    SUM(QTRETIRADA)  AS QTRETIRADA
                                 FROM SUBGRPRODFIL SG, PRSUBGRPROD AN,
                                    (SELECT
                                       CDFILIAL, CDPRODUTO , CDSERVFILI, DTCARDAPIO, QTCOMEFETCAR, QTCLIEFETCAR,QTRETIRADA,
                                       CUSTO_PRODUTO(CDFILIAL, CDPRODUTO, V_NRTIPOCUSTO, DTCARDAPIO, SYSDATE) * QTRETIRADA AS VRCUSTPROD
                                     FROM
                                       (SELECT
                                          CA.CDFILIAL, CA.CDSERVFILI, CA.DTCARDAPIO, CDALMOXARIFE, CDLOCALESTOQ,
                                          NRLOTEESTQ, NRSUBLOTE, CA.QTCOMEFETCAR, CA.QTCLIEFETCAR, IR.CDPRODUTO,
                                          SUM(IR.QTRETIRADA * PRA.VRFATOCONV) AS QTRETIRADA
                                       FROM
                                         CARDADIA CA, ITLANCTOEST IL,
                                         ITEMRETI IR, PRODUTO PRA
                                       WHERE
                                          (CA.CDFILIAL    = P_CDFILIAL )                      AND
                                          (IL.CDFILIAL    = C_RETIRADAPLA.CDFILIALLANC)       AND
                                          (IL.NRLANCESTQ  = C_RETIRADAPLA.NRLANCESTQ)         AND
                                          (IL.CDFILIAL    = IR.CDFIITLANEST)                  AND
                                          (IL.NRLANCESTQ  = IR.NRLANCESTQ)                    AND
                                          (IL.NRSEQUITEM  = IR.NRSEQUITEM)                    AND
                                          (NVL(IL.IDEVOLESTOQ,'S') = 'S')                              AND
                                          (CA.CDFILIAL   = IR.CDFILIAL)                       AND
                                          (CA.NRCARDAPIO = IR.NRCARDAPIO)                     AND
                                          (PRA.CDPRODUTO = IR.CDPRODUTO)
                                       GROUP BY
                                          CA.CDFILIAL,CA.CDSERVFILI,CA.DTCARDAPIO,CA.QTCOMEFETCAR,
                                          CA.QTCLIEFETCAR, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
                                          IR.CDPRODUTO))  R
                                 WHERE
                                      SG.CDFILISUBGR   = V_CDFILISUBGR
                                  AND SG.CDGRUPPROD    = V_CDGRUPPROD
                                  AND SG.CDFILISUBGR = AN.CDFILISUBGR
                                  AND SG.CDGRUPPROD  = AN.CDGRUPPROD
                                  AND SG.CDSUBGRPROD = AN.CDSUBGRPROD
                                  AND R.CDPRODUTO    = AN.CDPRODUTO
                                 GROUP BY
                                    R.CDFILIAL, DTCARDAPIO, CDSERVFILI, SG.CDGRUPPROD, AN.CDSUBGRPROD, DECODE( SG.IDSUBGRMATPRIMA, 'S', 'M', 'P')) LOOP

             /*Inclui Item d Retirada*/
             V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

             INSERT INTO ITLANCROPFILIAL
                (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                 NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP,
                 CDGRUPPRODROP, CDSUBGRPRODROP, IDTPSUBGRUPPROD)
             VALUES
                (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_RETIRADAPLA.CDFILIALLANC, C_RETIRADAPLA.NRLANCESTQ,
                 '', C_ITRETIRADAPLA.CDSERVFILI,  C_ITRETIRADAPLA.QTRETIRADA, C_ITRETIRADAPLA.VRCUSTPROD,
                 C_ITRETIRADAPLA.CDGRUPPROD, C_ITRETIRADAPLA.CDSUBGRPROD, C_ITRETIRADAPLA.IDSUBGRMATPRIMA);
          END LOOP;
      END LOOP;
   END IF;

/* Vendas For-Sale */

  IF V_IDGERROPFOS  = 'S' THEN
     V_NRITLANCROP := '00000';
     V_DTLANCROP := '01/01/1990';

    FOR C_REG IN( SELECT CDFILIAL,DTENTRVENDA,
                         NRNUMERNF,
                         SUM(VRVENDBRUT) VRVENDAS,
                         SUM(VRACRESCIMO) VRACRESCIMO,
                         SUM(VRDESCONTO) VRDESCONTOS
                       FROM(SELECT V.CDFILIAL, V.DTENTRVENDA,V.CDLOJA,V.CDCAIXA,
                                  SUM (TRUNC(I.QTPRODVEND *(NVL(I.VRUNITVEND,0) + NVL(I.VRUNITVENDCL,0)),2)) AS VRVENDBRUT,
                                  DECODE(V.IDSITUVENDA,'O',SUM (NVL(I.VRACRITVEND,0)),0) AS VRACRESCIMO,
                                  DECODE(V.IDSITUVENDA,'O',SUM (NVL(I.VRDESITVEND,0)),0) AS VRDESCONTO,
                                  SUM (I.QTPRODVEND) AS QTVENDIDA,
                                  0 AS NRNUMERNF

                             FROM VENDA V INNER JOIN ITEMVENDA I
                                                ON  (I.CDFILIAL    = V.CDFILIAL   )
                                                AND (I.CDCAIXA     = V.CDCAIXA    )
                                                AND (I.NRSEQVENDA  = V.NRSEQVENDA )

                                          INNER JOIN CAIXA C
                                                ON (V.CDFILIAL  = C.CDFILIAL)
                                                AND (V.CDCAIXA  = C.CDCAIXA)
                                                AND (V.CDLOJA   = C.CDLOJA)
                         WHERE (V.CDFILIAL  = P_CDFILIAL)
                           AND (V.DTENTRVENDA BETWEEN V_DTINIC AND P_DTFINAL)
                         GROUP BY V.CDFILIAL,V.DTENTRVENDA,V.IDSITUVENDA ,V.CDLOJA,V.CDCAIXA)
                         GROUP BY CDFILIAL,DTENTRVENDA,NRNUMERNF
                 )LOOP
         /*Valor de itens cancelados (Pode haver vendas concluidas,porem com itens cancelados)*/
         BEGIN
         SELECT NVL(SUM(VRITCANCEL),0) VRVENDBRUT
           INTO V_ITVENDACAN
             FROM(SELECT V.CDFILIAL, V.DTENTRVENDA, V.CDLOJA, V.CDCAIXA,
                    SUM (TRUNC(C.QTPRODVENDC * (NVL(C.VRUNITVENDC,0) + NVL(C.VRUNITVENCLC,0)),2)) AS VRITCANCEL
               FROM VENDA V INNER JOIN ITVENDACAN C
                                  ON  (C.CDFILIAL    = V.CDFILIAL   )
                                  AND (C.CDCAIXA     = V.CDCAIXA    )
                                  AND (C.NRSEQVENDA  = V.NRSEQVENDA )

                            INNER JOIN CAIXA C
                                  ON (V.CDFILIAL  = C.CDFILIAL)
                                  AND (V.CDCAIXA  = C.CDCAIXA)
                                  AND (V.CDLOJA   = C.CDLOJA)

               WHERE (V.CDFILIAL    = P_CDFILIAL)
                 AND (V.DTENTRVENDA = C_REG.DTENTRVENDA)
               GROUP BY V.CDFILIAL,V.DTENTRVENDA,V.IDSITUVENDA,V.CDLOJA,V.CDCAIXA);
          EXCEPTION
              WHEN NO_DATA_FOUND THEN
                  V_ITVENDACAN := 0;
              END;

         /*Valor de vendas canceladas*/
         BEGIN
            SELECT NVL(SUM(VRITCANCEL),0)
            INTO V_VRVENDACANCEL
              FROM (SELECT V.CDFILIAL, V.DTENTRVENDA, V.CDLOJA, V.CDCAIXA,
                           SUM (TRUNC(I.QTPRODVEND *(NVL(I.VRUNITVEND,0) + NVL(NVL(I.VRUNITVENDCL,0),0)),2))AS VRITCANCEL,
                           SUM (I.QTPRODVEND) AS QTVENDIDA
                      FROM VENDA V INNER JOIN ITEMVENDA I
                                         ON (I.CDFILIAL    = V.CDFILIAL   )
                                        AND (I.CDCAIXA     = V.CDCAIXA    )
                                        AND (I.NRSEQVENDA  = V.NRSEQVENDA )

                                  INNER JOIN CAIXA C
                                        ON  (V.CDFILIAL  = C.CDFILIAL)
                                        AND (V.CDCAIXA  = C.CDCAIXA )
                                        AND (V.CDLOJA   = C.CDLOJA)

               WHERE (V.CDFILIAL    = P_CDFILIAL)
                 AND (V.DTENTRVENDA = C_REG.DTENTRVENDA)
                 AND (V.IDSITUVENDA = 'C')
               GROUP BY V.CDFILIAL,V.DTENTRVENDA,V.IDSITUVENDA ,V.CDLOJA,V.CDCAIXA);
          EXCEPTION
            WHEN NO_DATA_FOUND THEN
                V_VRVENDACANCEL := 0;
          END;

          V_VENDABRUTA     := C_REG.VRVENDAS + V_ITVENDACAN;
          V_VRCANCELAMENTO := V_VRVENDACANCEL + V_ITVENDACAN;
          V_VRCONTABIL     := V_VENDABRUTA - C_REG.VRDESCONTOS - V_VRCANCELAMENTO;

      V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
      IF V_DTLANCROP <> C_REG.DTENTRVENDA THEN
        V_NRROPFILIAL := NULL;
        SELECT MAX(NRROPFILIAL)
        INTO V_NRROPFILIAL
        FROM ROPFILIAL
        WHERE CDFILIAL    = P_CDFILIAL AND
              DTFILIALROP = C_REG.DTENTRVENDA;
        V_DTLANCROP   := C_REG.DTENTRVENDA;

        IF V_NRROPFILIAL IS NULL THEN
           V_NRROPFILIAL := V_NRROPFILINI;
        END IF;

        SELECT MAX(NRLANCROP)
          INTO V_NRLANCROP
          FROM LANCROPFILIAL
         WHERE CDFILIAL = P_CDFILIAL AND
               NRROPFILIAL = V_NRROPFILIAL;
        IF V_NRLANCROP IS NULL THEN
           V_NRLANCROP  := 0;
           V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
        ELSE
           V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
        END IF;
     END IF;

     V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');
     INSERT INTO LANCROPFILIAL
      (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP)
     VALUES
      (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_REG.DTENTRVENDA, 'FF');
     INSERT INTO ITLANCROPFILIAL
      (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, VRITEMLANCROP, VRDESCITLANCROP, VRBRUTITLANCROP, VRCANCITLANCROP,
        NRLANCTONFITROP)
     VALUES
      (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP,V_VRCONTABIL ,C_REG.VRDESCONTOS,V_VENDABRUTA ,V_VRCANCELAMENTO,
      C_REG.NRNUMERNF );
    END LOOP;
   END IF;

   /*Lanamentos de Despesas Diversas*/
   IF V_IDDESPDIVROP = 'S' THEN
      V_NRITLANCROP := '00000';
      V_DTLANCROP := '01/01/1990';
      FOR C_DESPESASDIV IN (SELECT
                               D.CDFILIAL, D.CDDESPPREV, D.DTPRDESPFILI,
                               D.VRPRDESPFILI, D.VRREDESPFILI, IDDESTACAROP, E.CDPRODUTO
                            FROM
                               DESPPREVFILI D, DESPPREV E
                            WHERE
                               (D.CDFILIAL     = P_CDFILIAL)          AND
                               (D.CDDESPPREV   = E.CDDESPPREV)        AND
                               (D.DTPRDESPFILI BETWEEN V_DTINIC -32 AND P_DTFINAL)) LOOP

  /* Verifica se a despesa j existe na tabela*/
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM LANCROPFILIAL L, ITLANCROPFILIAL I
             WHERE
               L.CDFILIAL =  C_DESPESASDIV.CDFILIAL AND
               L.DTLANCROP = C_DESPESASDIV.DTPRDESPFILI AND
               CDITEMLANCROP = C_DESPESASDIV.CDPRODUTO AND
               L.CDFILIAL =  I.CDFILIAL AND
               L.NRROPFILIAL = I.NRROPFILIAL AND
               L.NRLANCROP = I.NRLANCROP AND
               IDTPLANCROP = DECODE(C_DESPESASDIV.IDDESTACAROP,  'S','CD','DD');
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
            END;

         IF V_COUNT = 0 THEN

            V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
            IF V_DTLANCROP <> C_DESPESASDIV.DTPRDESPFILI THEN

               V_NRROPFILIAL := NULL;
               SELECT MAX(NRROPFILIAL)
               INTO V_NRROPFILIAL
               FROM ROPFILIAL
               WHERE CDFILIAL    = P_CDFILIAL AND
                     DTFILIALROP = C_DESPESASDIV.DTPRDESPFILI;


               V_DTLANCROP   := C_DESPESASDIV.DTPRDESPFILI;
               IF V_NRROPFILIAL IS NULL THEN
                  V_NRROPFILIAL := V_NRROPFILINI;
               END IF;
               /* PEGA O ULTIMO NUMERO DO LANAMENTO */
               SELECT MAX(NRLANCROP)
                 INTO V_NRLANCROP
                 FROM LANCROPFILIAL
                WHERE CDFILIAL = P_CDFILIAL AND
                      NRROPFILIAL = V_NRROPFILIAL;

               IF V_NRLANCROP IS NULL THEN
                  V_NRLANCROP  := 0;
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               ELSE
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               END IF;
            END IF;

            /*Inclui Despesas Diversas*/

            IF C_DESPESASDIV.IDDESTACAROP = 'S' THEN
               V_CONSUMO := V_CONSUMO + C_DESPESASDIV.VRREDESPFILI;
            END IF;

            INSERT INTO LANCROPFILIAL
               (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP,CDFILCENTROP)
            VALUES
               (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_DESPESASDIV.DTPRDESPFILI, DECODE(C_DESPESASDIV.IDDESTACAROP,  'S','CD','DD'),
                '', '',  '', '', C_DESPESASDIV.CDFILIAL, '', V_CDFILCENTPME);

            V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

            INSERT INTO ITLANCROPFILIAL
               (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP, VRBRUTITLANCROP)
            VALUES
               (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_DESPESASDIV.CDFILIAL, '',
                '', C_DESPESASDIV.CDPRODUTO,  1, C_DESPESASDIV.VRREDESPFILI, C_DESPESASDIV.VRPRDESPFILI);
         END IF;
      END LOOP;
   END IF;


   IF V_IDDESPMOBROP= 'S' THEN
   /*Lanamentos de Despesas com Mo-de-Obra*/
      V_DTLANCROP := '01/01/1990';
      V_NRITLANCROP := '00000';

      FOR C_DESPESASMO IN (SELECT L.CDFILIAL, L.DTLANCDESFAT, L.CDDESPADMFAT,
                                  L.VRLANCDESFAT, L.VRTXLANCDESP, L.VRPREVDESFAT,DECODE(V_IDOBRINFNRFUNMO,'S',NVL(L.NRFUNCFILIFAT,0),0)NRFUNCFILIFAT
                           FROM LANCDESPFATU L
                           WHERE (L.CDFILIAL = P_CDFILIAL)                           AND
                                 (L.DTLANCDESFAT BETWEEN (V_DTINIC-32)  AND P_DTFINAL)) LOOP


         /* Verifica se a despesa j existe na tabela*/
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM LANCROPFILIAL L, ITLANCROPFILIAL I
             WHERE
               L.CDFILIAL =  C_DESPESASMO.CDFILIAL AND
               L.DTLANCROP = C_DESPESASMO.DTLANCDESFAT AND
               CDFILIALNFITROP = C_DESPESASMO.CDFILIAL AND
               CDITEMLANCROP = C_DESPESASMO.CDDESPADMFAT AND
               L.CDFILIAL =  I.CDFILIAL AND
               L.NRROPFILIAL = I.NRROPFILIAL AND
               L.NRLANCROP = I.NRLANCROP AND
               IDTPLANCROP = 'DM';
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
            END;

         IF V_COUNT = 0 THEN
            V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
            IF V_DTLANCROP <> C_DESPESASMO.DTLANCDESFAT THEN

               V_NRROPFILIAL := NULL;
               SELECT MAX(NRROPFILIAL)
               INTO V_NRROPFILIAL
               FROM ROPFILIAL
               WHERE CDFILIAL    = P_CDFILIAL AND
                     DTFILIALROP = C_DESPESASMO.DTLANCDESFAT;


               V_DTLANCROP   := C_DESPESASMO.DTLANCDESFAT;
               IF V_NRROPFILIAL IS NULL THEN
                  V_NRROPFILIAL := V_NRROPFILINI;
               END IF;
            /* PEGA O ULTIMO NUMERO DO LANAMENTO */
               SELECT MAX(NRLANCROP)
                 INTO V_NRLANCROP
                 FROM LANCROPFILIAL
                WHERE CDFILIAL = P_CDFILIAL AND
                      NRROPFILIAL = V_NRROPFILIAL;

               IF V_NRLANCROP IS NULL THEN
                  V_NRLANCROP  := 0;
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               ELSE
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               END IF;
            END IF;

            /*Inclui Despesas Mo de Obra*/
            INSERT INTO LANCROPFILIAL
               (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP)
            VALUES
               (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_DESPESASMO.DTLANCDESFAT, 'DM', '',
                '',  '', '', C_DESPESASMO.CDFILIAL, '', V_CDFILCENTPME);


            V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

            INSERT INTO ITLANCROPFILIAL
               (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
                NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP, VRBRUTITLANCROP)
            VALUES
               (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_DESPESASMO.CDFILIAL, '',
                '', C_DESPESASMO.CDDESPADMFAT,  C_DESPESASMO.NRFUNCFILIFAT, C_DESPESASMO.VRLANCDESFAT, C_DESPESASMO.VRPREVDESFAT );
         END IF;

      END LOOP;
   END IF;


   IF V_IDDESPENTROP = 'S' THEN
      V_DTLANCROP := '01/01/1990';
      V_NRLANCTONF := 'W';

      FOR C_DESPCOMPRA IN (SELECT N.CDTIPOOPER, N.NRNOTAFISC, N.CDFILIAL, N.NRLANCTONF,
                                  DTENTRSAID,   N.CDSERIE, N.CDFORNECED, CDFILIRECE,
                                  N.IDSTATUS,   N.DTINIREFEFAT, N.DTFINREFEFAT,
                                  I.CDPRODUTO, I.NRSEQITPROD,  I.VRBRUTITEM AS VALORTOTAL,
                                  I.QTITEMPROD AS QUANTIDADE, M.NUMERORE
                           FROM TIPOOPERACROP A, NOTAFISCAL N, ITEMPRODUTO I,
                                (
                                 SELECT P.CDPRODUTO
                                   FROM PRODUTO P,
                                         ( SELECT CDPRODUTO
                                             FROM PRODUTOROP
                                            WHERE CDFILIAL = V_FILIALPAR  AND
                                                  IDTPPRODUTOROP = 'DE') R
                                  WHERE
                                     P.CDARVPROD BETWEEN R.CDPRODUTO AND RPAD( R.CDPRODUTO , 10 , 'Z') AND
                                     LENGTH(P.CDARVPROD) = (SELECT LENGTH(REPLACE(CDPICTPROD,'.','')) FROM PARAMGERAL )
                                 MINUS
                                 SELECT P.CDPRODUTO
                                   FROM PRODUTO P,
                                         ( SELECT CDPRODUTO
                                             FROM PRODUTOROP
                                            WHERE CDFILIAL = V_FILIALPAR  AND
                                                  IDTPPRODUTOROP = 'EX') R
                                  WHERE
                                     P.CDARVPROD BETWEEN R.CDPRODUTO AND RPAD( R.CDPRODUTO , 10 , 'Z') AND
                                     LENGTH(P.CDARVPROD) = (SELECT LENGTH(REPLACE(CDPICTPROD,'.','')) FROM PARAMGERAL )) P,

                                (SELECT NRNUMERNF AS NUMERORE, CDFILIAL, DTNUMERNF AS DATARE
                                    FROM NUMERORNF
                                   WHERE CDFILIAL = P_CDFILIAL
                                     AND IDENTRSAID = 'E'
                                     AND DTNUMERNF BETWEEN V_DTINIC AND P_DTFINAL) M
                           WHERE
                              (N.CDFILIRECE   = P_CDFILIAL)                            AND
                              (N.IDENTRSAIDOP = 'E')                                   AND
                              (N.DTENTRSAID BETWEEN P_DTINICIAL  AND P_DTFINAL)        AND
                              (V_FILIALPAR    = A.CDFILIAL)                            AND
                              (N.CDTIPOOPER   = A.CDTIPOOPER)                          AND
                              (N.IDENTRSAIDOP = A.IDENTRSAIDOP)                        AND
                              (N.CDFILIAL      = M.CDFILIAL(+))                        AND
                              (N.DTENTRSAID    = M.DATARE(+))                          AND
                              (N.NRSEQCAPADOC  = M.NUMERORE(+))                        AND
                              (A.IDTPOPERACROP = 'DE')                                 AND
                              (N.IDSTATUS    <> 'C')                                   AND
                              (N.IDSISTEMA NOT IN ('PAG','REC','FIS','CMP') OR N.IDSISTEMA IS NULL) AND
                              (I.CDFILIAL   = N.CDFILIAL) AND
                              (I.NRLANCTONF = N.NRLANCTONF) AND
                              (P.CDPRODUTO  = I.CDPRODUTO)
                           ORDER BY N.CDFILIAL, N.NRLANCTONF )LOOP

         IF V_NRLANCTONF <> C_DESPCOMPRA.NRLANCTONF THEN

            V_NRLANCTONF := C_DESPCOMPRA.NRLANCTONF;
            V_NRLANCROP := LPAD(V_NRLANCROP + 1,10,'0');
            V_NRITLANCROP := '00000';

            IF V_DTLANCROP <> C_DESPCOMPRA.DTENTRSAID THEN

               V_NRROPFILIAL := NULL;
               SELECT MAX(NRROPFILIAL)
               INTO V_NRROPFILIAL
               FROM ROPFILIAL
               WHERE CDFILIAL    = P_CDFILIAL AND
                     DTFILIALROP = C_DESPCOMPRA.DTENTRSAID;


               V_DTLANCROP   := C_DESPCOMPRA.DTENTRSAID;
               IF V_NRROPFILIAL IS NULL THEN
                  V_NRROPFILIAL := V_NRROPFILINI;
               END IF;
             /* PEGA O ULTIMO NUMERO DO LANAMENTO */
               SELECT MAX(NRLANCROP)
                 INTO V_NRLANCROP
                 FROM LANCROPFILIAL
                WHERE CDFILIAL = P_CDFILIAL AND
                      NRROPFILIAL = V_NRROPFILIAL;

               IF V_NRLANCROP IS NULL THEN
                  V_NRLANCROP  := 0;
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               ELSE
                  V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
               END IF;
            END IF;

            /*Inclui Lanamento*/
            INSERT INTO LANCROPFILIAL
               (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
                NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP,    NRLANCTONFROP, CDFILCENTROP, IDSTATUSROP,
                DTINIREFEROP,  DTFINREFEROP, NUMEROREROP)
            VALUES
               (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_DESPCOMPRA.DTENTRSAID, 'DE', C_DESPCOMPRA.CDFORNECED,
                C_DESPCOMPRA.CDTIPOOPER,  C_DESPCOMPRA.NRNOTAFISC, C_DESPCOMPRA.CDSERIE, C_DESPCOMPRA.CDFILIAL,
                C_DESPCOMPRA.NRLANCTONF, V_CDFILCENTPME, C_DESPCOMPRA.IDSTATUS, C_DESPCOMPRA.DTINIREFEFAT,
                C_DESPCOMPRA.DTFINREFEFAT, C_DESPCOMPRA.NUMERORE);
         END IF;

/* verifica se existe alguma despesa diversa relacionada ao produto da nota fiscal para lanar este valor previsto de despesa

        SELECT
              NVL(SUM(D.VRPRDESPFILI),0)
          INTO V_VRPRDESPFILI
          FROM
             DESPPREVFILI D,
             ( SELECT P.CDPRODUTO, CDDESPPREV, nmdespprev
                 FROM PRODUTO P,
                     (SELECT CDPRODUTO, CDDESPPREV, nmdespprev FROM DESPPREV) R
               WHERE
                     P.CDARVPROD BETWEEN R.CDPRODUTO AND RPAD( R.CDPRODUTO , 10 , 'Z') AND
                     LENGTH(P.CDARVPROD) = (SELECT LENGTH(REPLACE(CDPICTPROD,'.','')) FROM PARAMGERAL )
              ) E
          WHERE
             (D.CDFILIAL     = P_CDFILIAL)          AND
             (D.CDDESPPREV   = E.CDDESPPREV)    AND
             (E.CDPRODUTO    = C_DESPCOMPRA.CDPRODUTO )AND
             TO_CHAR(D.DTPRDESPFILI, 'MM/YYYY') =  TO_CHAR( C_DESPCOMPRA.DTENTRSAID, 'MM/YYYY') AND
             NOT EXISTS (SELECT A.CDDESPPREV
                           FROM ITLANCROPFILIAL I, LANCROPFILIAL L,
                               (SELECT P.CDPRODUTO, CDDESPPREV
                                  FROM PRODUTO P,
                                    (SELECT CDPRODUTO, CDDESPPREV FROM DESPPREV) R
                                 WHERE
                                    P.CDARVPROD BETWEEN R.CDPRODUTO AND RPAD( R.CDPRODUTO , 10 , 'Z') AND
                                    LENGTH(P.CDARVPROD) = (SELECT LENGTH(REPLACE(CDPICTPROD,'.','')) FROM PARAMGERAL )) A
                          WHERE
                             (L.CDFILIAL       = P_CDFILIAL)                        AND
                             (L.IDTPLANCROP    = 'DE')                              AND
                             (L.DTLANCROP      = C_DESPCOMPRA.DTENTRSAID)           AND
                             (L.CDFILIAL       = I.CDFILIAL)                        AND
                             (L.NRROPFILIAL    = I.NRROPFILIAL)                     AND
                             (L.NRLANCROP      = I.NRLANCROP)                       AND
                              I.CDFILIALNFITROP = C_DESPCOMPRA.CDFILIAL AND
                             (I.NRLANCTONFITROP = C_DESPCOMPRA.NRLANCTONF) AND
                             I.CDITEMLANCROP   = A.CDPRODUTO) ;  */

         /*Inclui Item do Lanamento*/
         V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

         INSERT INTO ITLANCROPFILIAL
            (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
             NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP, VRBRUTITLANCROP)
         VALUES
            (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_DESPCOMPRA.CDFILIAL, C_DESPCOMPRA.NRLANCTONF,
             C_DESPCOMPRA.NRSEQITPROD, C_DESPCOMPRA.CDPRODUTO,  C_DESPCOMPRA.QUANTIDADE, C_DESPCOMPRA.VALORTOTAL, 0);
      END LOOP;
   END IF;

   /*Custo padrao do servio principal sesso de PU*/
   V_DTLANCROP := '01/01/1990';
   V_NRITLANCROP := '00000';
   FOR C_CUSTPADR IN (SELECT C.CDFILIAL, DTREFCUSPADR, SUM(C.VRCUSTPADR) AS VRCUSTPDR
                        FROM CUSTPADR C, SERVICO S
                       WHERE (C.CDFILIAL     = P_CDFILIAL)
                         AND (C.DTREFCUSPADR BETWEEN V_DTINIC  AND P_DTFINAL)
                         AND (S.CDFILIAL     = C.CDFILIAL)
                         AND (S.CDSERVFILI   = C.CDSERVFILI)
                         AND (S.IDSERVPRINC  = 'S')
                       GROUP BY C.CDFILIAL, DTREFCUSPADR ) LOOP

      V_NRLANCROP:= LPAD(V_NRLANCROP + 1,10,'0');
      IF V_DTLANCROP <> C_CUSTPADR.DTREFCUSPADR THEN

         V_NRROPFILIAL := NULL;
         SELECT MAX(NRROPFILIAL)
           INTO V_NRROPFILIAL
           FROM ROPFILIAL
          WHERE CDFILIAL    = P_CDFILIAL AND
                DTFILIALROP = C_CUSTPADR.DTREFCUSPADR;


         V_DTLANCROP   := C_CUSTPADR.DTREFCUSPADR;
         IF V_NRROPFILIAL IS NULL THEN
            V_NRROPFILIAL := V_NRROPFILINI;
         END IF;
/* PEGA O ULTIMO NUMERO DO LANAMENTO */
         SELECT MAX(NRLANCROP)
           INTO V_NRLANCROP
           FROM LANCROPFILIAL
          WHERE CDFILIAL = P_CDFILIAL AND
                NRROPFILIAL = V_NRROPFILIAL;

         IF V_NRLANCROP IS NULL THEN
            V_NRLANCROP  := 0;
            V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
         ELSE
            V_NRLANCROP  := LPAD(V_NRLANCROP + 1,10,'0');
         END IF;
      END IF;


      INSERT INTO LANCROPFILIAL
         (CDFILIAL, NRROPFILIAL,  NRLANCROP,  DTLANCROP, IDTPLANCROP, CDCLIENTEROP, CDTIPOOPERROP,
          NRNOTAFISCROP,    CDSERIENFROP,    CDFILIALNFROP, NRLANCTONFROP, CDFILCENTROP)
      VALUES
         (P_CDFILIAL, V_NRROPFILIAL, V_NRLANCROP, C_CUSTPADR.DTREFCUSPADR, 'PP', '',
          '',  '', '', C_CUSTPADR.CDFILIAL, '', V_CDFILCENTPME);


      V_NRITLANCROP := LPAD(V_NRITLANCROP + 1,5,'0');

      INSERT INTO ITLANCROPFILIAL
         (CDFILIAL, NRROPFILIAL, NRLANCROP, NRITLANCROP, CDFILIALNFITROP, NRLANCTONFITROP,
          NRSEQITPRODROP, CDITEMLANCROP,QTITEMLANCROP,    VRITEMLANCROP)
      VALUES
         (P_CDFILIAL,  V_NRROPFILIAL, V_NRLANCROP, V_NRITLANCROP, C_CUSTPADR.CDFILIAL, '',
          '', '',  1, C_CUSTPADR.VRCUSTPDR);

   END LOOP;

   DELETE FROM RELAT_MOVIEST
   WHERE
      (CDFILIMOVIEST = P_CDFILIAL)      AND
      (NRRELATMOVI   = V_NRMOVIESTQ);


   /* Apurao do Consumo -  Perodo Consumo = Considera Consumo Anterior */

   V_DTINICIO := V_DTINIC;
   V_DTFINAL  := V_DTINIC;

   IF (V_IDPERCONSROP = 'P') AND  (V_NRMESCONSROP > 1) AND (P_IDCONSANT = 'N')  THEN
      V_NRCOUNT   := 1;
      WHILE (V_NRCOUNT < V_NRMESCONSROP) LOOP

         SELECT '01/'||DECODE(TO_CHAR(V_DTINICIO, 'MM') , '01', '12/'||TO_CHAR(TO_NUMBER(TO_CHAR(V_DTINICIO, 'YYYY'))-1), LPAD(TO_NUMBER(TO_CHAR(V_DTINICIO, 'MM')) - 1, 2,'0')||'/'|| TO_CHAR(V_DTINICIO, 'YYYY'))
         INTO V_DTINICIO
         FROM DUAL;
         V_DTFINAL := LAST_DAY(TO_DATE(V_DTINICIO));

         V_NRCONS := 0;
         SELECT COUNT(*)
         INTO V_NRCONS
         FROM ROPFILIAL
         WHERE
            CDFILIAL    = P_CDFILIAL  AND
            DTFILIALROP BETWEEN V_DTINICIO AND V_DTFINAL;
         IF NVL(V_NRCONS,0)  = 0 THEN
            APURAC_RESOPERFILI(P_CDFILIAL, V_DTINICIO, V_DTFINAL, 'S');
         END IF;

         V_NRCOUNT := V_NRCOUNT + 1;
      END LOOP;
   END IF;

   IF V_IDFECHDIAROP = 'S'  THEN
      SELECT COUNT(*)
        INTO V_COUNT
        FROM APURACAOROP
       WHERE CDFILIAL = P_CDFILIAL;

      IF V_COUNT > 0 THEN
         UPDATE APURACAOROP
         SET DTINIAPURACROP  = P_DTFINAL + 1,
             DTFINAPURACROP  = P_DTFINAL + 1
         WHERE CDFILIAL = P_CDFILIAL;
      ELSE
         INSERT INTO APURACAOROP ( CDFILIAL, DTINIAPURACROP, DTFINAPURACROP)
                           VALUES( P_CDFILIAL, (P_DTFINAL + 1), (P_DTFINAL + 1));
      END IF;
   END IF;
END;

/
-- VERSAO --
BEGIN ATUVEROBJ('APURAC_RESOPERFILI','6.28.038'); END;
/
